angular.module('goldcar')
    .directive('whenReady', ['$interpolate', function($interpolate) {
        return {
            restrict: 'A',
            priority: Number.MIN_SAFE_INTEGER, // execute last, after all other directives if any.
            link: function($scope, $element, $attributes) {
                var expressions = $attributes.whenReady.split(';');
                var waitForInterpolation = false;
                var hasReadyCheckExpression = false;

                function evalExpressions(expressions) {
                    expressions.forEach(function(expression) {
                        $scope.$eval(expression);
                    });
                }

                if ($attributes.whenReady.trim().length === 0) { return; }

                if ($attributes.waitForInterpolation && $scope.$eval($attributes.waitForInterpolation)) {
                    waitForInterpolation = true;
                }

                if ($attributes.readyCheck) {
                    hasReadyCheckExpression = true;
                }

                if (waitForInterpolation || hasReadyCheckExpression) {
                    requestAnimationFrame(function checkIfReady() {
                        var isInterpolated = false;
                        var isReadyCheckTrue = false;

                        if (waitForInterpolation && $element.text().indexOf($interpolate.startSymbol()) >= 0) { // if the text still has {{placeholders}}
                            isInterpolated = false;
                        }
                        else {
                            isInterpolated = true;
                        }

                        if (hasReadyCheckExpression && !$scope.$eval($attributes.readyCheck)) { // if the ready check expression returns false
                            isReadyCheckTrue = false;
                        }
                        else {
                            isReadyCheckTrue = true;
                        }

                        if (isInterpolated && isReadyCheckTrue) { evalExpressions(expressions); }
                        else { requestAnimationFrame(checkIfReady); }

                    });
                }
                else {
                    evalExpressions(expressions);
                }
            }
        };
    }])

    .directive('dynamic', function ($compile) {
        return {
            restrict: 'A',
            replace: true,
            link: function (scope, ele, attrs) {
                scope.$watch(attrs.dynamic, function (html) {
                    if(typeof html !== 'undefined' && html.indexOf('###') >= 0){
                        html = '<b style="font-family: \'montserrat-semibold\';">' + html.replace('###', '</b>. ');
                    }
                    if(typeof html !== 'undefined' && html.indexOf('<a') >= 0 && html.indexOf('</a>') >= 0){
                        var start = html.indexOf('href') + 6;
                        var end;
                        var url;
                        var urlFinal;
                        for(var i = start; i < html.length; i++){
                            if(html[i] == '"') end = i;
                        }
                        url = html.substring(start, end);
                        if(url.indexOf('http') < 0 && url.indexOf('https') < 0) urlFinal = 'http://' + url;
                        else urlFinal = url;
                        html = html.replace('href="' + url, 'ng-click="abrirNavegadorPopup(\'' + urlFinal + '\')');
                    }
                    ele.html(html);
                    $compile(ele.contents())(scope);
                });
            }
        };
    })

    .directive('compile', ['$compile', function ($compile) {
        return function(scope, element, attrs) {
            scope.$watch(
                function(scope) {
                    return scope.$eval(attrs.compile);
                },
                function(value) {
                    element.html(value);
                    $compile(element.contents())(scope);
                }
            )};
    }])

    .directive('onFinishRender', function ($timeout) {
        return {
            restrict: 'A',
            link: function (scope, element, attr) {
                if (scope.$last === true) {
                    $timeout(function () {
                        scope.$emit('ngRepeatFinished');
                    });
                }
            }
        }
    })

    .directive('customCheckbox', function() {
        return {
            restrict: 'E',
            template: '<div class="custom-checkbox"></div>',
            replace: true,
            link: function(scope, element, attrs, model) {

                scope.toggle = function(evento) {

                    console.log('OK');
                    if ( angular.element(evento.target).hasClass('checked') ) {

                        angular.element(evento.target).removeClass('checked');
                    } else {

                        angular.element(evento.target).addClass('checked');
                    }
                }
            }
        };
    })

    .directive('ngClick', function ($timeout) {
        var delay = 500;

        return {
            restrict: 'A',
            priority: -1,
            link: function (scope, elem) {
                var disabled = false;

                function onClick(evt) {
                    if (disabled) {
                        evt.preventDefault();
                        evt.stopImmediatePropagation();
                    } else {
                        disabled = true;
                        $timeout(function () { disabled = false; }, delay, false);
                    }
                }

                scope.$on('$destroy', function () { elem.off('click', onClick); });
                elem.on('click', onClick);
            }
        };
    })

    .directive('fancySelect', ['$ionicModal', function ($ionicModal) {
        return {
            restrict: 'E',
            templateUrl: 'templates/configuracion/fancy-select.html',
            scope: {
                'items': '=',
                'text': '=',
                'value': '=',
                'callback': '&'
            },

            link: function (scope, element, attrs) {
                scope.multiSelect = attrs.multiSelect === 'true' ? true : false;
                scope.allowEmpty = attrs.allowEmpty === 'false' ? false : true;

                scope.headerText = attrs.headerText || '';
                scope.defaultText = scope.text || '';

                $ionicModal.fromTemplateUrl('templates/configuracion/fancy-select-items.html', {
                    'scope': scope
                }).then(function (modal) {
                    scope.modal = modal;
                });

                scope.showItems = function (event) {
                    event.preventDefault();
                    scope.modal.show();
                };

                scope.hideItems = function () {
                    scope.modal.hide();
                };

                scope.$on('$destroy', function () {
                    scope.modal.remove();
                });

                scope.validateSingle = function (item) {
                    scope.text = item;
                    scope.value = item;

                    scope.hideItems();

                    if (typeof scope.callback == 'function') {
                        scope.callback ( { value: item } );
                    }
                }
            }
        };
    }]

);
