angular.module('goldcar')
    .filter('translate', function ($rootScope) {
        return function (translateString) {
            if( typeof translateString === "undefined" || translateString == "" ){
                if( $rootScope.config.traduccionesDebug ) return '???';
                else return '';
            }
            else return translateString;
        };
    })

    .filter('format', function () {
        return function (input) {
            var args = arguments;
            return input.replace(/\{(\d+)\}/g, function (match, capture) {
                return args[1*capture + 1];
            });
        };
    })

    .filter('trusted', ['$sce', function ($sce) {
        return function(url) {
            return $sce.trustAsResourceUrl(url);
        };
    }])

    .filter('sinTildes', function(){
        return function(input){
            if(typeof input !== 'undefined' && input != null && input != '') {
                var tittles = "ÃÀÁÄÂÈÉËÊÌÍÏÎÒÓÖÔÙÚÜÛãàáäâèéëêìíïîòóöôùúüûÑñÇç";
                var original = "AAAAAEEEEIIIIOOOOUUUUaaaaaeeeeiiiioooouuuunncc";

                for (var i = 0; i < tittles.length; i++) {
                    input = input.replace(tittles.charAt(i), original.charAt(i)).toLowerCase();
                }
            }
            return input;
        };
    })

    .filter('passbook', function($rootScope){
        return function(apellido, bonoCliente, idReserva){
            var url = '';
            if ( typeof usuario !== 'undefined' && typeof reservaSeleccionada !== 'undefined' ) {
                var patron = $rootScope.config.patronPassbook;
                var encripted = sha256(reservaSeleccionada.idReserva + usuario + reservaSeleccionada.Bonocliente + patron);

                url = 'https://www.goldcar.es/passbook/gen/idreserva/' + reservaSeleccionada.idReserva + '/e/' + encripted;
            }
            return url;
        };
    })

    .filter('dateShow', function() {
        return function(date) {
            if(typeof date !== "undefined" && date != null) {
                return moment(date).format('DD MMMM');
            }
            return null;
        }
    })

    .filter('formatFechaDisp', function() {
        return function(date) {
            if(typeof date !== "undefined" && date != null) {
                return moment(date).format('D MMM');    
            }
            return null;
        }
    })
    
    .filter('filterHTML', function() {
        return function(texto, percentage, maxFee, divisa) {
            texto = texto.replace("@@VAR1@@", percentage);
            texto = texto.replace("@@VAR2@@", maxFee);
            texto = texto.replace("@@VAR3@@", divisa);
            return texto;
        }
    })

    .filter('filterPrecios', function() {
        return function(precio) {
            if(typeof precio !== 'undefined') {
                var formatNumber = {
                    separador: ".", // separador para los miles
                    sepDecimal: ',', // separador para los decimales
                    formatear: function (num){
                        num +='';
                        if(num.indexOf('.') >= 0) {
                            var splitStr = num.split('.');
                            var splitLeft = splitStr[0];
                            var splitRight = splitStr.length > 1 ? this.sepDecimal + splitStr[1] : '';
                            var regx = /(\d+)(\d{3})/;
                            while (regx.test(splitLeft)) {
                                splitLeft = splitLeft.replace(regx, '$1' + this.separador + '$2');
                            }
                            return splitLeft + splitRight.substring(0, 3);
                        }
                        else if (num.indexOf('.') < 0 && num.indexOf(',') < 0) return num + ',00';
                        else return num;
                    }
                }
                return formatNumber.formatear(precio);
            }
        }
    })

    .filter('filterPreciosDispo', function() {
        return function(precio) {
            if(typeof precio !== 'undefined') {
                var formatNumber = {
                    separador: ".", // separador para los miles
                    sepDecimal: ',', // separador para los decimales
                    formatear: function (num){
                        num +='';
                        var splitStr = num.split('.');
                        var splitLeft = splitStr[0];
                        var splitRight = splitStr.length > 1 ? this.sepDecimal + splitStr[1] : '';
                        var regx = /(\d+)(\d{3})/;
                        while (regx.test(splitLeft)) {
                            splitLeft = splitLeft.replace(regx, '$1' + this.separador + '$2');
                        }
                        return splitLeft + splitRight.substring(0, 3);
                    }
                }
                return formatNumber.formatear(precio);
            }
        }
    })
    
    .filter('formatHoraDisp', function() {
        return function(hour) {
            if(typeof hour !== "undefined" && hour != null) {
                return moment().set('hour', hour.split(':')[0]).set('minute', hour.split(':')[1]).format("HH:mm");
            }
            return null;
        }
    })

    .filter('IVADisponibilidad', function () {
        return function (iva) {
            if(typeof iva !== 'undefined' && parseFloat(iva) > 0) return parseFloat(iva)*100;
            else return 0;
        }
    })

    .filter('formatearMiles', function () {
        return function(num) {
            return num.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ".");
        }
    })

    // FILTRA LOS NOMBRES PARA LAS TARIFAS SELECCIONADAS
    .filter('filterTarifaName', function () {
        return function (tarifa) {
            if(typeof tarifa.reference !== 'undefined') return tarifa.descripcion;
            else if(typeof tarifa.reference === 'undefined' && tarifa.fuel == 'FullFuel') return 'Full/Full';
            else if(typeof tarifa.reference === 'undefined' && tarifa.fuel == 'FlexFuel') return 'Smart';
        }
    })
    
    .filter('ordenarDisponibles', ['$filter', function ($filter) {
        return function(categorias) {
            var nuevoOrden = [];
            var noDisponibles = [];
            var sinDisponibles = true;
            for(var i = 0; i < categorias.length; i++) {
                for(var l = 0; l < categorias[i].grupos.length; l++) {
                    if(categorias[i].grupos[l].disponible) {
                        sinDisponibles = false;
                        break;
                    }
                }
                if(!sinDisponibles) {
                    nuevoOrden.push(categorias[i]);
                } else {
                    noDisponibles.push(categorias[i]);
                }
                sinDisponibles = true;
            }
            if(nuevoOrden.length <= 0) {
                return false;
            } else {
                nuevoOrden = $filter('orderBy')(nuevoOrden, "orderApp");
                if(noDisponibles.length > 0) {
                    for(var s = 0; s < noDisponibles.length; s++) {
                        nuevoOrden.push(noDisponibles[s]);
                    }
                }
                return nuevoOrden;
            }
        }
    }]);
