angular.module('goldcar.services', [])

    .service('Language', function ($q, $rootScope, $cordovaGlobalization, $log) {

        return {
            get: function() {

                var defaultIdioma;

                for (var idioma in  $rootScope.config.idiomas){
                    defaultIdioma = idioma;
                    break;
                }

                $log.debug("Idioma default: ", defaultIdioma);

                var q = $q.defer();
                if(localStorage['lang']) {
                    $log.debug("Si hay localStorage['lang']: ", localStorage['lang']);
                    if( $rootScope.config.idiomas[localStorage['lang']] )
                        $rootScope.language = localStorage['lang'];
                    else
                        $rootScope.language = localStorage['lang'] = defaultIdioma;
                    q.resolve($rootScope.language);
                } else {
                    $log.debug("No hay localStorage['lang']");
                    if(navigator.globalization){
                        $log.debug("Si hay navigator.globalization: ", navigator.globalization);
                        $cordovaGlobalization.getPreferredLanguage().then(
                            function(result) {
                                var mobileLanguage = result.value;
                                mobileLanguage = mobileLanguage.substring(0,2);
                                $log.debug("resultado getPreferredLanguage(): ", result );
                                $log.debug("Preferred Language result: ", mobileLanguage );
                                $log.debug("$rootScope.config.idiomas: ", $rootScope.config.idiomas );
                                if( $rootScope.config.idiomas[mobileLanguage] )
                                    $rootScope.language = localStorage['lang'] = mobileLanguage;
                                else
                                    $rootScope.language = localStorage['lang'] = defaultIdioma;
                                q.resolve($rootScope.language);
                            },
                            function(error) {
                                $log.debug("Error getting preferred Language: ", error);
                                $rootScope.language = localStorage['lang'] = 'es';
                                q.resolve($rootScope.language);
                            });
                    }

                    else {
                        $log.debug("No existe navigator.globalization, ponemos 'es' por default");
                        $rootScope.language = localStorage['lang'] = 'es';
                        q.resolve($rootScope.language);
                    }
                }
                return q.promise;
            }
        }

    })

    .service('GoldcarService', function ($http, $sce, $log, $rootScope, $q, JsonService, $injector) {

        // var templateCache = $injector.get('OAUTHService');

        var url = "https://www.goldcar.es/api/v1"; // PROD
        // var url = "https://gcfrontend.pre.aws.goldcar.ws/api/v1"; // PRE
        // var url = "https://gcfrontend.test.aws.goldcar.ws/api/v1"; // DEV
        $http.defaults.headers.common['Accept'] = 'application/json';
        $http.defaults.headers.common['Content-Type'] = 'application/json';

        function getYoutubeUrlEmbed(youTubeUrl){
            var youTubeId;
            var regExp = /^.*(youtu.be\/|v\/|u\/\w\/|embed\/|watch\?v=|\&v=)([^#\&\?]*).*/;
            var match = youTubeUrl.match(regExp);

            if (match && match[2].length == 11) {
                youTubeId = match[2];
            } else {
                if(youTubeUrl.indexOf("playlist") < 0)
                    return '';
            }

            if(youTubeUrl.indexOf("playlist") >= 0){
                youTubeId = youTubeUrl.substring(youTubeUrl.indexOf('list=') + 5);
                return "https://www.youtube.com/embed/playlist?list=" + youTubeId;
            }
            else
                return "https://www.youtube.com/embed/" + youTubeId;
        }

        function makeExternalLinks(texto){
            if(texto !== null && texto !== ''){
                var regex = /target="_blank"/g;
                var newString = texto.replace(regex,'');
                regex = /href="([\S]+)"/g;
                newString = newString.replace(regex, "href=\"#\" onClick=\"window.open('$1', '_system', 'location=yes,shouldPauseOnSuspend=yes')\"");
                return newString;
            } else return '';
        }

        var parseReserva = function (reserva) {
            moment.locale($rootScope.config.mapeo_moment[$rootScope.language]);

            var Cobertura = false;
            var ModeloGarantizado = false;

            reserva.FechaDay = moment(reserva.Fecha).format('LLLL');
            reserva.FechaDay = reserva.FechaDay.substring(0, reserva.FechaDay.length - 6);
            reserva.FechaTime = moment(reserva.Fecha).format('LT');

            reserva.Pickupdate = moment(reserva.Pickupdate);
            reserva.PickupdateDay = reserva.Pickupdate.format('dddd') + ', ' + moment(reserva.Pickupdate).format('LL');
            reserva.PickupdateTime = reserva.Pickupdate.format('LT');

            reserva.Dropoffdate = moment(reserva.Dropoffdate);
            reserva.DropoffdateDay = reserva.Dropoffdate.format('dddd') + ', ' + moment(reserva.Dropoffdate).format('LL');
            reserva.DropoffdateTime = reserva.Dropoffdate.format('LT');

            var now = new Date().getTime();
            var diffHours = reserva.Pickupdate - now;
            var time = 24 * 60 * 60 * 1000;

            if ( reserva.Cancelable && ( reserva.Estado == 2 || reserva.Estado == 5 ) )
                reserva.BotonCancelar = true;
            else
                reserva.BotonCancelar = false;

            reserva.FaltaPagar = parseFloat(reserva.Total) - parseFloat(reserva.Prepagado);

            reserva.NombreModeloGarantizado = '';
            reserva.ModeloGarantizado = false;

            if (typeof(reserva.Pickupplace.Idzonaventa) != undefined && reserva.Pickupplace.Idzonaventa == 2)
                reserva.TipoImpuesto = "IGIC";
            else
                reserva.TipoImpuesto = "IVA";

            reserva.Tarifaincluida = false;
            reserva.Paquetezero = false;
            reserva.Paquetecobertura = false;
            reserva.PaqueteNoReembolsable = false;
            reserva.PaquetePremium = false;

            if (reserva.Extras) {
                for (var cont = 0; reserva.Extras[cont]; cont++) {
                    if (reserva.Extras[cont].Codigo == "CoverRelax" || reserva.Extras[cont].Codigo == "CoverSuperRelax")
                        Cobertura = true;
                    if (reserva.Extras[cont].Codigo == "ModeloGarantizado") {
                        var NombreModeloGarantizado = reserva.Extras[cont].Descripcion.split('-');
                        reserva.NombreModeloGarantizado = NombreModeloGarantizado[1];
                        reserva.ModeloGarantizado = true;
                    }
                }
            }

            if (reserva.Paquetes) {
                for (var cont2 = 0; reserva.Paquetes[cont2]; cont2++) {
                    if (reserva.Paquetes[cont2].Tarifaincluida) reserva.Tarifaincluida = true;
                    if (reserva.Paquetes[cont2].Reference == "15035" || reserva.Paquetes[cont2].Reference == "15036" || reserva.Paquetes[cont2].Reference == "15046" || reserva.Paquetes[cont2].Reference == "15047") reserva.Paquetezero = true;
                    if (reserva.Paquetes[cont2].Reference == "15086") reserva.Paquetecobertura = true;
                    if (reserva.Paquetes[cont2].Reference == "15091") reserva.PaqueteNoReembolsable = true;
                    if (reserva.Paquetes[cont2].Reference == "15084") reserva.PaquetePremium = true;
                }
            }

            // TRANSFER ENABLED?

            var transfer = JsonService.getOfficeByCode(reserva.Pickupplace.Codigo);

            if(transfer) {
                if(typeof transfer.busData !== 'undefined' || transfer.busData != null || transfer.busData.length <=0)
                    reserva.busData = false;
                else
                    reserva.busData = true;
            }

            else
                reserva.busData = false;

            return reserva;
        };

        var idiomaPrintCount = 0;
        var idioma = function() {

            var idioma = $rootScope.config.mapeo_api[$rootScope.language];
            if (idiomaPrintCount <= 0)
                $log.debug("services.js -> Idioma elegido (GoldcarService): ", idioma);
            idiomaPrintCount++;
            return idioma;
        };

        var parseaGrupo = function (grupo) {

            // console.log(grupo);

            var addtarifas = {};
            grupo.PackKeyngo = null;
            grupo.PackUnique = null;
            grupo.PackPremium = null;
            for (var tarifa in grupo.tarifas) {
                grupo.tarifas[tarifa].tarifa_id = tarifa;
                if ( grupo.tarifas[tarifa].tarifa_disponible ) {
                  if(tarifa === 'PackKeyngo') grupo.PackKeyngo = grupo.tarifas[tarifa];
                  else if(tarifa === 'PackUnique') grupo.PackUnique = grupo.tarifas[tarifa];
                  else if(tarifa === 'PackPremium' || tarifa === 'Pack15113') grupo.PackPremium = grupo.tarifas[tarifa];
                  else addtarifas[tarifa] = grupo.tarifas[tarifa];
                }
            }
            // console.log("grupo.tarifas", grupo.tarifas);
            grupo.precio_totalf = parseFloat(grupo.precio_totalf);
            grupo.precio_total_diariof = parseFloat(grupo.precio_total_diariof);
            grupo.tarifa_no_reembolsable = grupo.tarifa_no_reembolsable;
            if(grupo.tarifa_disponible) {
                addtarifas[grupo.tarifa] = {
                    NumKmLimited: grupo.NumKmLimited,
                    cancelable: grupo.cancelable,
                    fuel: $rootScope.limpiaTarifaId(grupo.tarifa),
                    precio: "" + grupo.precio,
                    iva: parseFloat(grupo.iva[0]),
                    precio_inicial: grupo.precio_inicial,
                    precio_totalf: parseFloat(grupo.preciof),
                    precio_total: "" + grupo.precio,
                    precio_total_diario: "" + grupo.precio_diario,
                    precio_total_diariof: parseFloat(grupo.precio_diariof),
                    tarifa_fuel: grupo.tarifa_fuel,
                    preciof: grupo.preciof,
                    tarifa_no_reembolsable: grupo.tarifa_no_reembolsable,
                    tarifa_disponible: grupo.tarifa_disponible,
                    tarifa_id: grupo.tarifa,
                    formas_pago: grupo.formas_pago
                };
            }

            var minTarifa = null;
            for(var idTarifa in addtarifas) {
                var tmpTarifa = addtarifas[idTarifa];
                tmpTarifa.fuel = $rootScope.limpiaTarifaId(idTarifa);
                if(tmpTarifa.tarifa_disponible) {
                  if(minTarifa == null) minTarifa = tmpTarifa;
                  else {
                    if(minTarifa.precio_totalf > tmpTarifa.precio_totalf) minTarifa = tmpTarifa;
                  }
                }
            }
            if(minTarifa == null && (grupo.PackKeyngo !== null || grupo.PackUnique !== null || grupo.PackPremium !== null)){
                if(grupo.PackKeyngo !== null && grupo.PackUnique !== null && grupo.PackPremium !== null){
                    if(grupo.PackKeyngo.precio_totalf < grupo.PackUnique.precio_totalf) minTarifa = grupo.PackKeyngo;
                    else minTarifa = grupo.PackUnique;
                } else {
                    if(grupo.PackKeyngo !== null) minTarifa = grupo.PackKeyngo;
                    else if (grupo.PackUnique !== null) minTarifa = grupo.PackUnique;
                    else if (grupo.PackPremium !== null) minTarifa = grupo.PackPremium;
                }
                minTarifa.esPaquete = true;
            }
            grupo.minTarifa = minTarifa;
            grupo.tarifas = addtarifas;

            // PONER DENTRO DE EXTOBLIGATORIOS LOS EXTRAS DE EXTRAS_DISPO //
            if(typeof grupo !== 'undefined' && typeof grupo.extras_dispo !== 'undefined'){
                for(var extra_dispo in grupo.extras_dispo[grupo.grupo]){
                    grupo.extObligatorios.push({
                        alias: extra_dispo,
                        precio_total: grupo.extras_dispo[grupo.grupo][extra_dispo].ALL.precio
                    });
                }
            }

            //CHAPA UNIQUE
            // if(grupo.disponible) {
            //     var mod2Tarifas = {};
            //     var packsAIgnorar = $rootScope.config.packsAIgnorar;
            //     for (var tarifa in grupo.tarifas) {
            //         if(
            //             (packsAIgnorar.indexOf(tarifa) < 0) //No tiene que ser ignorado
            //         )
            //         mod2Tarifas[tarifa] = grupo.tarifas[tarifa];
            //     }
            //     grupo.tarifas = mod2Tarifas;

            //     var disponible = false;
            //     for (var tarifa in grupo.tarifas) {
            //         disponible = true;
            //         break;
            //     }
            //     grupo.disponible = disponible;
            // }
            //FIN CHAPA UNIQUE

            // console.log(grupo);

            return grupo;
        };

        var loginTries = 0;

        return {
            login: function (user, passwd) {  // LOGIN USUARIOS

                var data = { user: user, passwd: passwd, 'referer': 'app' };

                return $http.post(url + '/login', data)
                    .success(function (data, status, headers) {
                        if(typeof data.Picture !== 'undefined' && data.Picture != "") {
                            var picture = data.Picture;
                            picture = picture.replace(/\?sz=\d*/g, '');
                            data.Picture = picture;
                        }
                        return data;
                    });
            },
            getUsuario: function () { // VER DATOS USUARIO

                var self = this;

                return $http.get(url + '/usuario')
                    .success(function (data, status, headers) {
                        if ( loginTries < 2 && typeof data.error !== 'undefined' ) {
                            loginTries++;
                            if ( typeof localStorage['user'] !== 'undefined' && typeof localStorage['passwd'] !== 'undefined' ) {
                                self.login(localStorage['user'], localStorage['passwd']).then(function() {
                                    self.getUsuario();
                                })
                            } else {
                                if ( localStorage['redSocial'] == 'facebook' ) {
                                    $injector.get('OAUTHService').FBLogin();
                                } else if ( localStorage['redSocial'] == 'GooglePlus' ) {
                                    $injector.get('OAUTHService').GPlusLogin();
                                }
                            }
                        } else {
                            return data;
                        }
                    })
                    .error(function(error) {
                        if ( loginTries < 2 && typeof error !== 'undefined' && typeof error.error !== 'undefined' ) {
                            loginTries++;
                            if ( typeof localStorage['user'] !== 'undefined' && typeof localStorage['passwd'] !== 'undefined' ) {
                                self.login(localStorage['user'], localStorage['passwd']).then(function() {
                                    self.getUsuario();
                                })
                            } else {
                                if ( localStorage['redSocial'] == 'facebook' ) {
                                    $injector.get('OAUTHService').FBLogin();
                                } else if ( localStorage['redSocial'] == 'GooglePlus' ) {
                                    $injector.get('OAUTHService').GPlusLogin();
                                }
                            }
                        } else {
                            return error.data;
                        }
                    });
            },
            postUsuario: function (data) {  // MODIFICAR USUARIO

                return $http.post(url + '/usuario', data)
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            filtrarBusqueda: function (input) {  // FILTRAR BUSQUEDA LISTA OFICINAS
                var idiomaUser = idioma();

                return $http.get(url + '/oficinas/search/q/' + input + '/' + idiomaUser)
                    .success(function(data, status, headers){
                        return data;
                    })
                    .error(function(data, status, headers){
                        return data;
                    });
            },
            filtrarBusquedaOneWay: function (codigoOficinaRecogida, input) {  // FILTRAR BUSQUEDA LISTA OFICINAS ONEWAY
                var idiomaUser = idioma();

                return $http.get(url + '/oficinas/onewaySearch/q/' + codigoOficinaRecogida + '/' + input + '/' + idiomaUser)
                    .success(function(data, status, headers){
                        return data;
                    })
                    .error(function(data, status, headers){
                        return data;
                    });
            },
            postRecuperarPasswd: function (mail) { // RECUPERAR CONTRASEÑA USUARIO

                var data = {

                    "email": mail
                };

                return $http.post(url + '/login/genpwd', data)
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getReservasActivas: function () { // VER RESERVAS ACTIVAS DEL USUARIO

                return $http.get(url + "/usuarioReservas/activas")
                    .success(function (data, status, headers) {
                        if ( data.data_empty != true ) {
                            if (data.length > 0) {
                                data.forEach(function (item) {

                                    if ( item.Incluye ) {
                                        if ( item.Incluye['UniquePunto1'] ) {

                                            var items = [];

                                            for ( incluye in item.Incluye ) {

                                                items.push(makeExternalLinks(item.Incluye[incluye][0]));
                                            }

                                            item.Incluye = items;
                                        } else {

                                            for (var i = 0; i < item.Incluye.length; i++)
                                                item.Incluye[i] = makeExternalLinks(item.Incluye[i]);
                                        }
                                    }

                                    if ( item.NoIncluye ) {

                                        for (var i = 0; i < item.NoIncluye.length; i++)
                                            item.NoIncluye[i] = makeExternalLinks(item.NoIncluye[i]);
                                    }
                                    parseReserva(item);
                                });
                            }
                            return data;
                        }
                        data.length = 0;
                        return data;
                    });
            },
            getReservasAnteriores: function () {  // VER RESERVAS ANTERIORES DEL USUARIO

                return $http.get(url + "/usuarioReservas/historial")
                    .success(function (data, status, headers) {
                        if ( data.data_empty != true ){
                            if (data.length > 0) {
                                data.forEach(function (item) {
                                    if(typeof item.Incluye !== 'undefined'){
                                        for(var i=0; i<item.Incluye.length; i++)
                                            item.Incluye[i] = makeExternalLinks(item.Incluye[i]);
                                    }
                                    if(typeof item.NoIncluye !== 'undefined'){
                                        for(var i=0; i<item.NoIncluye.length; i++)
                                            item.NoIncluye[i] = makeExternalLinks(item.NoIncluye[i]);
                                    }
                                    parseReserva(item);
                                });
                            }
                            return data;
                        }
                        data.length = 0;
                        return data;
                    });
            },
            getReservasCanceladas: function () {  // VER RESERVAS CANCELADAS DEL USUARIO

                return $http.get(url + "/usuarioReservas/canceladas")
                    .success(function (data, status, headers) {
                        if ( data.data_empty != true ) {
                            if (data.length > 0) {
                                data.forEach(function (item) {
                                    parseReserva(item);
                                });
                            }
                            return data;
                        }
                        data.length = 0;
                        return data;
                    });
            },
            getDetalleReserva: function (idReserva) { // VER DETALLES DE RESERVA

                return $http.get(url + "/reserva/id/q/" + idReserva)
                    .success(function (data, status, headers) {
                        if (data.length > 0) {
                            data.forEach(function (item) {
                                parseReserva(item);
                            });
                        }
                        return data;
                    });
            },
            cancelarReserva: function (reserva){
                return $http.put(url + '/reserva/cancelar', reserva)
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getOficinas: function () {  // VER TODAS LAS OFICINAS

                return $http.get(url + "/oficinas/orderAllOffices/q/" + idioma())
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getDetallesOficina: function (oficina) { // VER DATOS DE UNA OFICINA

                return $http.get(url + "/oficina/q/" + oficina + "/" + idioma())
                    .success(function (data, status, headers) {
                        var mapAndroid = "geo:?q=" + data.Coordenadas + "(" + data.Fullname + ")";
                        var mapIOS = "http://maps.apple.com?daddr=" + data.Coordenadas + "(" + data.Fullname + ")";
                        data.mapLink = ionic.Platform.isIOS() ? mapIOS : mapAndroid;

                        if(typeof data.Direccion !== 'undefined')
                            data.Direccion = makeExternalLinks(data.Direccion);
                        if(typeof data.Meetingpoint !== 'undefined')
                            data.Meetingpoint = makeExternalLinks(data.Meetingpoint);
                        if(typeof data.Linkyoutube !== 'undefined')
                            data.Linkyoutube = $sce.trustAsResourceUrl(getYoutubeUrlEmbed(data.Linkyoutube));
                        return data;
                    });
            },
            getTiempoOficina: function (idOficina) {  // VER DATOS DE UNA OFICINA

                return $http.get(url + "/oficina/weather/q/" + idOficina)
                    .success(function (data, status, headers) {
                        data.EnlaceMasDias = data.localidad.url_pronostico_15_dias;
                        data.TempMin = data.pronostico_dias.dia[0].temp_minima;
                        data.TempMax = data.pronostico_dias.dia[0].temp_maxima;
                        data.Viento = data.pronostico_dias.dia[0].viento;
                        data.ImagenEstado = data.pronostico_dias.dia[0].icono;
                        data.ImagenViento = data.pronostico_dias.dia[0].ico_viento;
                        return data;
                    });
            },
            postUsuarioClub: function (data) {  // REGISTRAR USUARIO EN GOLDCAR CLUB

                return $http.post(url + "/usuario", data)
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getPuntos: function () {  // VER PUNTOS DEL USUARIO

                return $http.get(url + "/usuario/puntos")
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            putRecuperarPuntos: function (fechaRecogida, IdReserva) { // RECUPERAR PUNTOS DEL USUARIO

                var data = {

                    "FechaRecogida": fechaRecogida, // 2016-01-06
                    "IdReserva": IdReserva // 12343214
                };

                return $http.put(url + "/usuario/recuperarpuntos", data)
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getPremios: function () { // VER PREMIOS DEL USUARIO

                return $http.get(url + "/usuario/catalogoregalos/q/" + idioma())
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            putPremio: function (idPremio, userData) {  // CANJEAR PREMIOS DEL USUARIO

                var id = { data: userData, id: idPremio };

                return $http.put(url + "/usuario/canjearpuntos/q/" + idioma(), id)

                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getCodigosPromo: function () { // RECUPERAR CODIGOS PROMOCIONALES DEL USUARIO

                return $http.get(url + "/usuario/mispremios/q/" + idioma())
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getPromociones: function () { // RECUPERAR LISTA DE PROMOCIONES

                return $http.get(url + "/usuario/promociones/q/" + idioma())
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getPromo: function (id) { // RECUPERAR INFORMACION DE UNA PROMOCION

                return $http.get(url + "/usuario/promocion/q/" + id + "/" + idioma())
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getFacturas: function () { // OBTENER FACTURAS DEL USUARIO

                return $http.get(url + "/usuario/misfacturas")
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            postSesion: function (data) { // ENVIAR DATOS DE SESION
                return $http.post(url + '/sesion', data);
            },
            getSesion: function () {  // RECUPERAR DATOS DE SESION
                return $http.get(url + '/sesion');
            },
            getOficinasDevol: function (codigoOficina) {  // RECUPERAR OFICINAS DISPONIBLES PARA DEVOLUCION

                return $http.get(url + '/oficinas/oneway/q/' + codigoOficina + '/' + idioma())
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getCalendarioOficina: function (codigoOficina, fecha, tipo) {  // RECUPERAR CALENDARIO DE UNA OFICINA

                fecha = moment(fecha).format('YYYY/MM/DD');

                var urlFinal;

                if ( tipo && tipo == 'recogida' ){
                    urlFinal = url + '/oficina/calendario/q/' + codigoOficina + '/' + fecha + '/' + idioma();
                }

                else if ( tipo && tipo == 'entrega' ){
                    urlFinal = url + '/oficina/calendario/q/' + codigoOficina + '/' + fecha + '/' + fecha + '/' + idioma();
                }

                return $http.get(urlFinal)
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getCalendario: function(codigoOficina, tipo) { // RECUPERAR CALENDARIO OFICINA INICIO

                return $http.get(url + '/oficina/calendario/q/' + codigoOficina + '/' + idioma())
                    .success(function (data, status, headers) {

                        return data;
                });
            },
            getPoliticaCancelacion: function(precioReserva) { // RECUPERAR POLITICA DE CANCELACION DE UNA RESERVA

                return $http.get(url + '/reserva/getCargoCancelacion/q/' + precioReserva)
                    .success(function (data, status, headers) {
                        return data;
                });
            },
            getHorarioOficina: function (codigoOficina, fecha, tipo) {  // RECUPERAR HORARIO DE UNA OFICINA

                var urlFinal;

                var fechaDevolucion = moment(fecha).format('YYYY/MM/DD');

                if ( typeof tipo !== 'undefined' && tipo == 'recogida' ){
                    urlFinal = url + '/oficina/horario/q/' + codigoOficina + '/' + fechaDevolucion;
                }

                else if ( typeof tipo !== 'undefined' && tipo == 'entrega' ){

                    // console.log("$rootScope.fechaRecogida", $rootScope.fechaRecogida);

                    var fechaRecogida = moment($rootScope.fechaRecogida);
                    fechaRecogida = fechaRecogida.format('YYYY/MM/DD');

                    var timeArray = $rootScope.horaRecogida;

                    timeArray = timeArray.split(':');
                    var horaRecogida = ( timeArray[0].length == 1 ) ? '0' + timeArray[0] : timeArray[0];
                    var minutosRecogida = ( timeArray[1].length == 1 ) ? '0' + timeArray[1] : timeArray[1];

                    urlFinal = url + '/oficina/horario/q/' + codigoOficina + '/' + fechaDevolucion + '/' + fechaRecogida + '/' + horaRecogida + '/' + minutosRecogida;
                }

                return $http.get(urlFinal)
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getDisponibilidad: function () {  // RECUPERAR DISPONIBILIDAD
                var deferred = $q.defer();
                $http.get(url + '/disponibilidad')
                    .then(function (result) {
                        // console.log("RESULT DISPONIBILIDAD", result);
                        if(!result.data.error && result.data) {
                            try{
                                result.data.forEach(function (grupo) {
                                    parseaGrupo(grupo);
                                });
                            }
                            catch(err){
                                Object.keys(result.data).forEach(function (grupo) {
                                    parseaGrupo(result.data[grupo]);
                                });
                            }
                            deferred.resolve(result.data);
                        }else{
                            deferred.reject(result.error);
                        }
                    }, function (e) {
                        deferred.reject(e);
                    });
                return deferred.promise;
            },
            getDisponibilidadGrupo: function () {  // RECUPERAR DISPONIBILIDAD DEL GRUPO DE SESION
            //   console.log("getDisponibilidadGrupo in Service.js");
              /*return $http.get(url + '/disponibilidad/grupo')
                  .success(function (data, status, headers) {
                      return data;
                  });
              */
                var deferred = $q.defer();
                $http.get(url + '/disponibilidad/grupo').then(function (result) {
                  console.log(result);
                    if(!result.data.error && result.data) {
                        parseaGrupo(result.data);
                        deferred.resolve(result.data);
                    }
                }, function (e) {
                    deferred.reject(e);
                });
                return deferred.promise;

            },
            getDatosGrupo: function (grupo) { // RECUPERAR DATOS DEL GRUPO SELECCIONADA
              //console.log("GRUPO EN SERVICE "+ grupo);
                return $http.get(url + '/disponibilidad/grupo')
                    .success(function (data, status, headers) {
                        // console.log("DATA", data);
                        return data;
                    });
            },
            getCodigoPromo: function (codigo) { // RECUPERAR CODIGO PROMOCIONAL

                return $http.get(url + '/disponibilidad/codigoPromocional/q/' + codigo)
                    .success(function (data, status, headers) {

                        return data;
                    })
                    .error(function(error) {

                        return error;
                    });
            },
            getDisponibilidadConGrupo: function (idReserva) { // RECUPERAR DATOS GRUPO DE RESERVA

                return $http.get(url + '/disponibilidad/grupo/q/' + idReserva)
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getActivarCodigoPromo: function (codigo) { // ACTIVAR CODIGO PROMOCIONAL

                return $http.get(url + '/disponibilidad/activarCodigoPromocional/q/' + codigo)
                    .success(function (data, status, headers) {

                        return data;
                    });
            },

            getSubirGrupo: function () {  // RECUPERAR DISPONIBILIDAD PARA SUBIR DE GRUPO
                return $http.get(url + '/disponibilidad/upgrade')
                    .success(function (data, status, headers) {
                        return data;
                    });
            },

            getCondicionesServicio : function(office, fechaRecogida, fechaEntrega, grupo) { // VER CONDICIONES SERVICIO SEGÚN FILTROS
                return $http.get(url + '/terminos/q/' + office + '/' + fechaRecogida + '/' + fechaEntrega + '/' + grupo + '/' + idioma())
                    .success(function (data, status, headers) {
                        data.texto = makeExternalLinks(data.texto);
                        return data;
                    });
            },
            getTerminos: function (oficina, fechaRecogida, fechaDevolucion, grupo) {  // RECUPERAR TERMINOS Y CONIDICIONES DE LA RESERVA

                return $http.get(url + '/terminos/q/' + oficina + '/' + fechaRecogida + '/' + fechaDevolucion + '/' + grupo + '/' + idioma())
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getTerminosSinReserva: function (oficina) {  // RECUPERAR TERMINOS Y CONIDICIONES DE LA RESERVA

                return $http.get(url + '/terminos/q/' + oficina + '/' + idioma())
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            postSolicitarReserva: function () {   // REALIZAR RESERVA CON DATOS DE SESION

                return $http.post(url + '/reserva')
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            putReserva: function (data) { // MODIFICAR RESERVA. PARA MODIFICAR UNA RESERVA, PRIMERO TENEMOS QUE RECOGER LOS DATOS DEL GRUPO "/disponibilidad/grupo"

                return $http.put(url + '/reserva', data)
                    .success(function (data, status, headers) {

                        return data;
                    })
            },
            postTarjetas: function () {

                return $http.post(url + '/usuario/tarjetas')
                    .success(function (data, status, headers) {

                        return data.data;
                })
            },
            postNuevaTarjeta: function (data) {

                return $http.post(url + '/usuario/nuevaTarjeta', data)
                    .success(function (data, status, headers) {

                    return data;
                })
            },
            postEliminarTarjeta: function (data) {

                return $http.post(url + '/usuario/eliminaTarjeta', data)
                    .success(function (data, status, headers) {

                    return data;
                })
            },
            postPrepagoDivisa: function (divisa) {

                return $http.post(url + '/reserva/prepagoDcc/q/' + divisa)
                    .success(function (data, status, headers) {

                    return data;
                })
            },
            setGdprTerms: function (type, status) {
                var data = {
                    'codigo': type,
                    'estado': status
                }

                return $http.post(url + '/sesion/usuarioCheckOptions', data)
                    .success(function (data, status, headers, config) {
                        return data;
                    })
                    .error(function (data, status, header, config) {
                        return data;
                    });
            },
            setCheckOptionsClub: function(checks){
                var data = {
                    checks: []
                }

                for(var l in checks){
                    data.checks.push({'codigo': l, 'estado': checks[l]});
                }

                // console.log(data);

                return $http.post(url + '/usuario/setCheckOptionsClub', data)
                    .success(function (data, status, header, config) {
                        return data;
                    })
                    .error(function (data, status, header, config){
                        return data;
                    });
            },
            unsubscribeUser: function(data) {
                return $http.post(url + '/unsubscribe', data)
                    .success(function (data, status, headers, config) {
                        return data;
                    })
                    .error(function (data, status, header, config) {
                        return data;
                    });
            },
            checkPagoReserva: function(idReserva) {
                return $http.get(url + '/reserva/checkPagoReserva/q/' + idReserva)
                    .success(function (data, status, headers, config) {
                        return data;
                    })
                    .error(function (data, status, header, config) {
                        return data;
                    });
            },
            // clientIdentificationType: function (email) {
            //     return $http.get(url + '/usuario/clientIdentifycationType/q/' + email)
            //         .success(function (data, status, header, config) {
            //             return data;
            //         })
            //         .error(function (data, status, header, config) {
            //             return data;
            //         })
            // },
            checkDataIntegrity: function (data) {
                return $http.post(url + '/reserva/checkDataIntegrity', data)
                    .success(function (data, status, header, config) {
                        return data;
                    })
                    .error(function (data, status, header, config) {
                        return data;
                    })
            },
            sendBookingSMS: function (phone, prefix, phoneNumber) {
                return $http.post(url + '/reserva/telefono/q/' + phone + '/' + idioma() + '/' + prefix + '/' + phoneNumber)
                    .success(function (data, status, header, config) {
                        return data;
                    })
                    .error(function (data, status, header, config) {
                        return data;
                    })
            },
            checkBookingSMS: function (code) {
                return $http.post(url + '/reserva/telefonocodigo/q/' + code + '/' + idioma())
                    .success(function (data, status, header, config) {
                        return data;
                    })
                    .error(function (data, status, header, config) {
                        return data;
                    })
            },
            mandatoryAddressBono: function (datosUsuario, datosReserva) {
                var strEnc = datosReserva.idReserva +
                    datosReserva.Pickupplace +
                    datosReserva.Idgrupo +
                    datosReserva.Idioma +
                    datosReserva.Idcliente +
                    "d924687d70e4570b145575d88a986f698d0e0cb7a6b544df18e7057264371d40";
                datosUsuario.strEnc = sha256(strEnc);

                console.log("strEnc", strEnc);
                console.log("datosUsuario", datosUsuario);
                return $http.post(url + '/reserva/mandatoryAddressBono', datosUsuario)
                    .success(function (data, status, header, config) {
                        return data;
                    })
                    .error(function (data, status, header, config) {
                        return data;
                    })
            }
            // setTyc: function () {
            //     return $http.post(url + '/sesion/tyc')
            //         .success(function (data, status, headers, config) {
            //             return data;
            //         })
            //         .error(function (data, status, header, config) {
            //             return data;
            //         });
            // }
        }
    })

    .service('JsonService', function ($filter, $http, $q, $log, $rootScope) {

        var preview = 0; // TODO: cambiar para produccion por 0 / desarrollo a 1
        var url = "https://gcdata.goldcar.es/api"; // PROD
        // var url = "https://gcdata.pre.aws.goldcar.ws/api"; // PRE
        // var url = "https://goldcardata.epigramdev.com/api"; // DEV

        $http.defaults.headers.common['Accept'] = 'application/json';

        var authHeaders = { 'X_GOLDCARDATA_USERNAME': 'GOLDCARDATA@API', 'X_GOLDCARDATA_PASSWORD': '54a56U43c6!73~' };

        var idiomaPrintCount = 0;
        var idioma = function() {

            var idioma = $rootScope.language;
            if (idiomaPrintCount <= 0)
                $log.debug("services.js -> Idioma elegido (JsonService): ", idioma);
            idiomaPrintCount++;
            return idioma;
        };

        return {
            getContent: function (content) {
                var contentName = "content_" + content;
                if (typeof localStorage[contentName] != "undefined")
                    return JSON.parse(localStorage[contentName]);
                else
                    return null;
            },
            getContentJSON: function (content, lang, lang_in, type) {

                var deferred = $q.defer();
                var headers = { 'X_GOLDCARDATA_USERNAME': 'GOLDCARDATA@API', 'X_GOLDCARDATA_PASSWORD': '54a56U43c6!73~' };
                var contentName = "content_" + content;
                if(!lang) lang = idioma();

                var forceControlIfModifiedAPI = 0;

                if ((typeof(localStorage[contentName]) != "undefined") &&  ((localStorage[contentName]) != null) &&  (typeof localStorage.getItem(contentName + '_lastModified') != "undefined") &&  (localStorage.getItem(contentName + '_lastModified') != null)){
                    headers["If-Modified-Since"] = localStorage.getItem(contentName + '_lastModified');
                    if ( $rootScope.config.forceControlIfModifiedAPI )
                        forceControlIfModifiedAPI = $rootScope.config.forceControlIfModifiedAPI;
                }

                var urlFinal = url + '/contents?handle=' + encodeURIComponent(content) + '&lang=' + encodeURIComponent(lang) + '&text=1&preview=' + preview;

                if(lang_in) urlFinal += '&lang-in=' + encodeURIComponent(lang_in);
                if(type) urlFinal += '&type=' + encodeURIComponent(type);
                urlFinal += '&forceControlIfModifiedAPI=' + encodeURIComponent(forceControlIfModifiedAPI);

                $http.get(urlFinal, {headers: headers})
                    .then(function (response) {
                        var data = null;

                        if (response.status == 200 && response.data) {
                            data = response.data;
                            localStorage.setItem(contentName, JSON.stringify(data));
                            localStorage.setItem(contentName + '_lastModified', response.headers('Last-Modified'));
                        }
                        else {
                            data = JSON.parse(localStorage.getItem(contentName));
                        }

                        deferred.resolve(data);
                    }, function (error) { // TODO : COMPROBAR QUE PASA LA PRIMERA VEZ QUE SE ABRE LA APP SIN LOCALSTORAGE !!!!

                        if((typeof(localStorage[contentName]) != "undefined") &&  ((localStorage[contentName]) != null)){

                            deferred.resolve(JSON.parse(localStorage[contentName]));
                        }
                        else{

                            deferred.reject(error);
                        }
                    });
                return deferred.promise;
            },
            getContentData: function (content) {

                var deferred = $q.defer();
                var headers = { 'X_GOLDCARDATA_USERNAME': 'GOLDCARDATA@API', 'X_GOLDCARDATA_PASSWORD': '54a56U43c6!73~' };
                var contentName = "content_" + content;
                var forceControlIfModifiedAPI = 0;

                if ((typeof(localStorage[contentName]) != "undefined") && ((localStorage[contentName]) != null) && (typeof localStorage.getItem(contentName + '_lastModified') != "undefined") && (localStorage.getItem(contentName + '_lastModified') != null)) {
                    headers["If-Modified-Since"] = localStorage.getItem(contentName + '_lastModified');
                    if ( $rootScope.config.forceControlIfModifiedAPI )
                        forceControlIfModifiedAPI = $rootScope.config.forceControlIfModifiedAPI;
                }

                var urlFinal = url + '/contents?handle=' + encodeURIComponent(content) + '&lang=' + encodeURIComponent(idioma()) + '&preview=' + preview + '&forceControlIfModifiedAPI=' + encodeURIComponent(forceControlIfModifiedAPI);

                $http.get(urlFinal, {headers: headers})
                    .then(function (response) {
                        var data = null;
                        if (response.status == 200 && response.data) {

                            data = response.data.data;
                            localStorage.setItem(contentName, JSON.stringify(data.content));
                            localStorage.setItem(contentName + '_lastModified', response.headers('Last-Modified'));
                        }
                        else {

                            data = JSON.parse(localStorage.getItem(contentName));
                        }
                        deferred.resolve(data);
                    }, function (error) {

                        if ((typeof(localStorage[contentName]) != "undefined") &&  ((localStorage[contentName]) != null)) {

                            var data = [];
                            data.content = JSON.parse(localStorage[contentName]);
                            deferred.resolve(data);
                        } else {

                            deferred.reject(error);
                        }
                    });
                return deferred.promise;
            },
            getFAQ: function () {

                var deferred = $q.defer();
                var headers = { 'X_GOLDCARDATA_USERNAME': 'GOLDCARDATA@API', 'X_GOLDCARDATA_PASSWORD': '54a56U43c6!73~' };

                var contentName = "content_faqs";

                var forceControlIfModifiedAPI = 0;

                if ((typeof(localStorage[contentName]) != "undefined") &&  ((localStorage[contentName]) != null) &&  (typeof localStorage.getItem(contentName + '_lastModified') != "undefined") &&  (localStorage.getItem(contentName + '_lastModified') != null)){
                    headers["If-Modified-Since"] = localStorage.getItem(contentName + '_lastModified');
                    if ( $rootScope.config.forceControlIfModifiedAPI )
                        forceControlIfModifiedAPI = $rootScope.config.forceControlIfModifiedAPI;
                }

                $http.get(url + '/faqs?lang=' + idioma() + '&forceControlIfModifiedAPI=' + encodeURIComponent(forceControlIfModifiedAPI), {headers: headers})
                    .then(function (response) {
                        var data = null;

                        if (response.status == 200 && response.data && response.data.items) {

                            data = response.data.items;
                            localStorage.setItem(contentName, JSON.stringify(data));
                            localStorage.setItem(contentName + '_lastModified', response.headers('Last-Modified'));
                        }
                        else {

                            data = JSON.parse(localStorage.getItem(contentName));
                        }

                        deferred.resolve(data);
                    }, function (error) {

                        if ((typeof(localStorage[contentName]) != "undefined") &&  ((localStorage[contentName]) != null)) {

                            var data = JSON.parse(localStorage[contentName]);
                            $log.debug("localStorage FAQS parsed: ", data);
                            deferred.resolve(data);
                        } else {

                            deferred.reject(error);
                        }
                    });
                return deferred.promise;
            },
            getPress: function () {
                var deferred = $q.defer();
                return $http.get(url + '/news?lang=' + idioma() + '&type=gcpress', {headers: authHeaders})
                    .then(function (response) {

                        deferred.resolve(response);

                        return deferred.promise;
                    }, function (response) {

                        deferred.reject(response);

                        return deferred.promise;
                    });
            },
            getOffices: function () {
                var deferred = $q.defer();
                return $http.get(url + '/offices?lang=' + idioma(), {headers: authHeaders})
                    .then(function (response) {

                        deferred.resolve(response);

                        return deferred.promise;
                    }, function (response) {

                        deferred.reject(response);

                        return deferred.promise;
                    });
            },
            getOfficeByCode: function (code) {
                var office = false;
                if($rootScope.todasLasOficinas) {
                    $rootScope.todasLasOficinas.forEach(function (pais) {
                        if(typeof pais.offices !== 'undefined'){
                            pais.offices.forEach(function (oficina) {
                                if(oficina.code == code){
                                    office = oficina;
                                    return;
                                }
                            });

                            if(office) return;
                        }
                    });
                }
                return office;
            },
            getOffice: function (code) {
                //  return;
                  var deferred = $q.defer();
                  return $http.get(url + '/offices/' + code + '?lang=' + idioma(), {headers: authHeaders})
                      .then(function (response) {

                          deferred.resolve(response.data);

                          return deferred.promise;
                      }, function (response) {

                          deferred.reject(response);

                          return deferred.promise;
                      });
            },
            getOfficesByCountry: function () {
                var deferred = $q.defer();
                var headers = { 'X_GOLDCARDATA_USERNAME': 'GOLDCARDATA@API', 'X_GOLDCARDATA_PASSWORD': '54a56U43c6!73~' };
                var contentName = "content_offices";

                var forceControlIfModifiedAPI = 0;

                if ((typeof(localStorage[contentName]) != "undefined") &&  ((localStorage[contentName]) != null) &&  (typeof localStorage.getItem(contentName + '_lastModified') != "undefined") &&  (localStorage.getItem(contentName + '_lastModified') != null)){
                    headers["If-Modified-Since"] = localStorage.getItem(contentName + '_lastModified');
                    if ( $rootScope.config.forceControlIfModifiedAPI )
                        forceControlIfModifiedAPI = $rootScope.config.forceControlIfModifiedAPI;
                }

                $http.get(url + '/offices/groupByCountry?lang=' + idioma() + '&forceControlIfModifiedAPI=' + encodeURIComponent(forceControlIfModifiedAPI), {headers: headers})
                    .then(function (response) {
                        var data = null;

                        if (response.status == 200 && response.data && response.data.data && typeof response.data.data.items !== 'undefined') {
                            data = response.data.data.items;
                            localStorage.setItem(contentName, JSON.stringify(data));
                            localStorage.setItem(contentName + '_lastModified', response.headers('Last-Modified'));
                        }
                        else {
                            data = JSON.parse(localStorage.getItem(contentName));
                        }

                        deferred.resolve(data);
                    }, function (error) {

                        if (localStorage[contentName] !== "undefined" && localStorage[contentName] != null) {

                            var data = JSON.parse(localStorage[contentName]);
                            deferred.resolve(data);
                        } else {

                            deferred.reject(error);
                        }
                    });
                return deferred.promise;
            },
            getBanners: function (tags) {
                var deferred = $q.defer();
                var headers = { 'X_GOLDCARDATA_USERNAME': 'GOLDCARDATA@API', 'X_GOLDCARDATA_PASSWORD': '54a56U43c6!73~' };
                var contentName = "content_banners_generales";
                var lang = idioma();
                var forceControlIfModifiedAPI = 0;
                var urlFinal;

                if(tags != null)
                    contentName = "content_banners_" + tags;

                if ((typeof(localStorage[contentName]) != "undefined") &&  ((localStorage[contentName]) != null) &&  (typeof localStorage.getItem(contentName + '_lastModified') != "undefined") &&  (localStorage.getItem(contentName + '_lastModified') != null)){
                    headers["If-Modified-Since"] = localStorage.getItem(contentName + '_lastModified');
                    if ( $rootScope.config.forceControlIfModifiedAPI )
                        forceControlIfModifiedAPI = $rootScope.config.forceControlIfModifiedAPI;
                }

                if( typeof lang === "undefined" )
                    lang = 'en';

                if(tags != null)
                    urlFinal = url + '/banners?lang=' + lang + '&type=gcapp&tags=' + tags + '&forceControlIfModifiedAPI=' + encodeURIComponent(forceControlIfModifiedAPI);

                else
                    urlFinal = url + '/banners?lang=' + lang + '&type=gcapp' + '&forceControlIfModifiedAPI=' + encodeURIComponent(forceControlIfModifiedAPI);

                $http.get(urlFinal, {headers: headers})
                    .then(function (response) {
                        var data = null;

                        if (response.status == 200 && response.data) {

                            data = response.data.items;
                            localStorage.setItem(contentName, JSON.stringify(data));
                            localStorage.setItem(contentName + '_lastModified', response.headers('Last-Modified'));
                        }
                        else {

                            data = JSON.parse(localStorage.getItem(contentName));
                        }

                        deferred.resolve(data);
                    }, function (error) {

                        if ((typeof(localStorage[contentName]) != "undefined") &&  ((localStorage[contentName]) != null)) {

                            var data = JSON.parse(localStorage[contentName]);
                            deferred.resolve(data);
                        } else {

                            deferred.reject(error);
                        }
                    });
                return deferred.promise;
            },
            getBackgrounds: function (tags) {
                var deferred = $q.defer();
                var headers = { 'X_GOLDCARDATA_USERNAME': 'GOLDCARDATA@API', 'X_GOLDCARDATA_PASSWORD': '54a56U43c6!73~' };
                var contentName = "content_backgrounds";
                var lang = idioma();
                var forceControlIfModifiedAPI = 0;
                var urlFinal;

                if(tags != null)
                    contentName = "content_backgrounds" + tags;

                if ((typeof(localStorage[contentName]) != "undefined") &&  ((localStorage[contentName]) != null) &&  (typeof localStorage.getItem(contentName + '_lastModified') != "undefined") &&  (localStorage.getItem(contentName + '_lastModified') != null)){
                    headers["If-Modified-Since"] = localStorage.getItem(contentName + '_lastModified');
                    if ( $rootScope.config.forceControlIfModifiedAPI )
                        forceControlIfModifiedAPI = $rootScope.config.forceControlIfModifiedAPI;
                }

                if( typeof lang === "undefined" )
                    lang = 'en';

                if(tags != null)
                    urlFinal = url + '/banners?lang=' + lang + '&type=gcapp&tags=' + tags + '&forceControlIfModifiedAPI=' + encodeURIComponent(forceControlIfModifiedAPI);

                else
                    urlFinal = url + '/banners?lang=' + lang + '&type=gcapp' + '&forceControlIfModifiedAPI=' + encodeURIComponent(forceControlIfModifiedAPI);

                $http.get(urlFinal, {headers: headers})
                    .then(function (response) {
                        var data = null;

                        if (response.status == 200 && response.data) {

                            data = response.data.items;
                            localStorage.setItem(contentName, JSON.stringify(data));
                            localStorage.setItem(contentName + '_lastModified', response.headers('Last-Modified'));
                        }
                        else {

                            data = JSON.parse(localStorage.getItem(contentName));
                        }

                        deferred.resolve(data);
                    }, function (error) {

                        if ((typeof(localStorage[contentName]) != "undefined") &&  ((localStorage[contentName]) != null)) {

                            var data = JSON.parse(localStorage[contentName]);
                            deferred.resolve(data);
                        } else {

                            deferred.reject(error);
                        }
                    });
                return deferred.promise;
            },
            getDescubreData: function (tags) {
                var deferred = $q.defer();
                var headers = { 'X_GOLDCARDATA_USERNAME': 'GOLDCARDATA@API', 'X_GOLDCARDATA_PASSWORD': '54a56U43c6!73~' };

                var contentName = "content_banners_" + tags;

                var forceControlIfModifiedAPI = 0;

                if ((typeof(localStorage[contentName]) != "undefined") &&  ((localStorage[contentName]) != null) &&  (typeof localStorage.getItem(contentName + '_lastModified') != "undefined") &&  (localStorage.getItem(contentName + '_lastModified') != null)){
                    headers["If-Modified-Since"] = localStorage.getItem(contentName + '_lastModified');
                    if ( $rootScope.config.forceControlIfModifiedAPI )
                        forceControlIfModifiedAPI = $rootScope.config.forceControlIfModifiedAPI;
                }

                $http.get(url + '/banners?lang=' + idioma() + '&type=gcapp-descubre&anyTags=' + tags + '&forceControlIfModifiedAPI=' + encodeURIComponent(forceControlIfModifiedAPI), {headers: headers})
                    .then(function (response) {
                        var data = null;

                        if (response.status == 200 && response.data) {

                            data = response.data.items;
                            localStorage.setItem(contentName, JSON.stringify(data));
                            localStorage.setItem(contentName + '_lastModified', response.headers('Last-Modified'));
                        }
                        else {

                            data = JSON.parse(localStorage.getItem(contentName));
                        }

                        deferred.resolve(data);
                    }, function (error) {

                        if ((typeof(localStorage[contentName]) != "undefined") &&  ((localStorage[contentName]) != null)) {

                            var data = JSON.parse(localStorage[contentName]);
                            deferred.resolve(data);
                        } else {

                            deferred.reject(error);
                        }
                    });
                return deferred.promise;
            },
            getTicket: function (office, service, ticketId, centerType, serviceExpress) {
                var deferred = $q.defer();
                $log.debug("getTicket office", office);
                $log.debug("getTicket service", service);
                $log.debug("getTicket ticketID", ticketId);
                var data = {
                    office: office,
                    lang: idioma()
                };
                if (service) data.service = service;
                if (ticketId) data.ticketId = ticketId;
                else {
                    data.centerType = centerType;
                    data.serviceExpress = serviceExpress;
                }
                $log.debug("getTicket data", data);
                return $http.post(url + '/virtualTickets/waitingFor', data, {headers: authHeaders})
                    .then(function (response) {
                        deferred.resolve(response);
                        $log.debug("getTicket response", response);
                        return deferred.promise;
                    }, function (response) {
                        deferred.reject(response);
                        $log.error("getTicket error response", response);
                        return deferred.promise;
                    });
            },
            createTicket: function (data) {
                data.lang = idioma();
                return $http.post(url + '/virtualTickets/create', data, {headers: authHeaders})
                    .success(function (data, status, headers) {

                    });
            },
            postEncuesta: function (data) {
                data.lang = idioma();
                return $http.post(url + '/surveys/satisfaction', data, {headers: authHeaders})
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            getAllTickets: function (data) {
                data.lang = idioma();
                return $http.post(url + '/virtualTickets/waitingFor', data, {headers: authHeaders})
                    .success(function (data, status, headers) {

                    });
            },
            postMail: function (data) {
                return $http.post(url + '/mails/send', data, {headers: authHeaders})
                    .success(function (data, status, headers) {

                        return data;
                    });
            },
            postSolicitarFacturas: function (data) {
                return $http.post(url + '/goldcarAPI/obtenerListaFacturas', data, {headers: authHeaders})
                    .success(function (data, status, headers) {

                    });
            },
            postListaConcursos: function (type, limit, offset) {
                var deferred = $q.defer();
                var lang = idioma();
                var data = null;

                if ( typeof type === 'undefined' ) {

                    type = '';
                }

                if ( typeof limit === 'undefined' ) {

                    limit = '';
                }

                if ( typeof offset === 'undefined' ) {

                    offset = '';
                }

                return $http.post(url + '/competitions?lang=' + lang + '&type=' + type + '&limit=' + limit + '&offset=' + offset, data, {headers: authHeaders})
                    .then(function (response) {

                        deferred.resolve(response.data);
                        return deferred.promise;
                    }, function (response) {

                        deferred.reject(response);
                        return deferred.promise;
                    });
            },
            postListaPremios: function (type, limit, offset) {
                var deferred = $q.defer();
                var lang = idioma();
                var data = null;

                $log.debug("Type ", type, "Limit ", limit, "Offset ", offset);

                if ( typeof type === 'undefined' ) {

                    type = '';
                }

                if ( typeof limit === 'undefined' ) {

                    limit = '';
                }

                if ( typeof offset === 'undefined' ) {

                    offset = '';
                }

                $log.debug("Type ", type, "Limit ", limit, "Offset ", offset);

                return $http.post(url + '/competitions?lang=' + lang + '&type=' + type + '&limit=' + limit + '&offset=' + offset, data, {headers: authHeaders})
                    .then(function (response) {

                        deferred.resolve(response);
                        $log.debug("post ListaConcursos", response);
                        return deferred.promise;
                    }, function (response) {

                        deferred.reject(response);
                        $log.error("post ListaConcursos", response);
                        return deferred.promise;
                    });
            }
        }

    })

    .service('OAUTHService', function ($filter, $http, $log, $cordovaFacebook, $rootScope, $state, $ionicLoading, $q, GoldcarService, Session, $ionicPopup, $cordovaOauth) {
        $http.defaults.headers.common['Accept'] = 'application/json';

        return {

            ////////////////////////////////
            //  ACCESO MEDIANTE GOOGLE +  //
            ////////////////////////////////

            GPlusLogin: function() { // The login function walks the user through the Google Auth process.
                var service = this;

                window.plugins.googleplus.login(
                    {
                        'scopes': 'profile email https://www.googleapis.com/auth/plus.login https://www.googleapis.com/auth/plus.profile.agerange.read',      // optional, space-separated list of scopes, If not included or empty, defaults to `profile` and `email`.
                        'webClientId': '589349113195-qpdppnbqvnut0ehtv3fv7cghtcipc166.apps.googleusercontent.com',                                                         // optional clientId of your Web application from Credentials settings of your project - On Android, this MUST be included to get an idToken. On iOS, it is not required.
                        'offline': false                                                            // optional, but requires the webClientId - if set to true the plugin will also return a serverAuthCode, which can be used to grant offline access to a non-Google server
                    },
                    function (success) {

                        service.GPlusMe(success.accessToken);
                    },
                    function (error) {
                        $log.debug('Login error: ', error);
                        $ionicLoading.hide();
                    }
                );
            },

            GPlusSilentLogin: function () { // You can call trySilentLogin to check if they're already signed in to the app and sign them in silently if they are. If it succeeds you will get the same object as the login function gets, but if it fails it will not show the authentication dialog to the user.
                var deferred = $q.defer();
                var service = this;

                $ionicLoading.show({
                    template: '<div class="cargando"><i style="font-weight: bold; margin-bottom: 5px;">' + $filter("translate")($rootScope.translations.loading.entrando) + '</i><br><ion-spinner icon="bubbles" class="spinner-positive"></ion-spinner></div>'
                });

                console.log("GPlusSilentLogin");
                window.plugins.googleplus.trySilentLogin(
                    {
                        'scopes': 'profile email https://www.googleapis.com/auth/plus.login https://www.googleapis.com/auth/plus.profile.agerange.read',      // optional, space-separated list of scopes, If not included or empty, defaults to `profile` and `email`.
                        'webClientId': "589349113195-qpdppnbqvnut0ehtv3fv7cghtcipc166.apps.googleusercontent.com",                                                         // optional clientId of your Web application from Credentials settings of your project - On Android, this MUST be included to get an idToken. On iOS, it is not required.
                        'offline': false                                                            // optional, but requires the webClientId - if set to true the plugin will also return a serverAuthCode, which can be used to grant offline access to a non-Google server
                    },
                    function (success) {
                        console.log(success);
                        service.GPlusMe(success.accessToken);
                        deferred.resolve(success);
                        $ionicLoading.hide();
                    },
                    function (error) {
                        $log.debug('SilentLogin error: ', error);
                        service.GPlusLogin();
                        deferred.reject(error);
                        $ionicLoading.hide();
                    }
                );
                return deferred.promise;
            },

            GPlusMe: function(accessToken) { // This function retrieves all the data that we have permissions for
                var service = this;
                this.getGooglePlusData(accessToken)
                    .then(function (result) {
                        $log.debug("User data: ", result.data);
                        result.data.accessToken = accessToken;
                        service.GPlusSendDataGoldcar(result.data);
                    }, function (error) {
                        $log.debug('Consulta datos fallida: ', error);
                        service.GPlusLogin();
                    });
            },

            GPlusSendDataGoldcar: function(data) {
                var regex = /\?sz=\d*/g;
                var picture = data.image.url.replace(regex, '');

                var resolvedData = {
                    'accessToken': data.accessToken,
                    // 'age_range': data.ageRange.min,
                    'email': data.emails[0].value,
                    'first_name': data.givenName,
                    'gender': '',
                    'last_name': data.familyName,
                    'locale': data.language,
                    'name': data.displayName,
                    'nombreRS': 'google',
                    'picture': picture,
                    'timezone': '',
                    'updated_time': '',
                    'usuarioIdent': data.id,
                    'verified': data.verified,
                    'referer': 'app'
                };

                // FOR WIDGET
                $rootScope.resolvedDataRRSS = resolvedData;

                this.loginWithRRSS(resolvedData)
                    .then(function (result) {

                        console.log("resultRRSS", result);

                        localStorage['user'] = result.data.Email;
                        localStorage['redSocial'] = 'GooglePlus';
                        localStorage['nombreUsuario'] = $rootScope.nombreUsuario = result.data.Nombre;
                        localStorage['apellidosUsuario'] = $rootScope.apellidosUsuario = result.data.Apellidos;
                        localStorage['puntosClub'] = $rootScope.puntosClub = result.data.Puntosclub;
                        localStorage['imagenPerfil'] = $rootScope.imagenPerfil = result.data.Picture;
                        localStorage['isLogged'] = $rootScope.isLogged = true;
                        $rootScope.datosUsuario = result.data;

                        Session.set({

                            "Lang": localStorage['lang'],
                            "nombre": $rootScope.datosUsuario.Nombre,
                            "apellidos": $rootScope.datosUsuario.Apellidos,
                            "user": $rootScope.datosUsuario.user,
                            "authenticated": $rootScope.datosUsuario.Authenticated
                        }).then(function (result) {

                        });

                        if( typeof data.cover !== 'undefined' && typeof data.cover.coverPhoto !== 'undefined' && typeof data.cover.coverPhoto.url !== 'undefined' )
                            localStorage['imagenCover'] = $rootScope.profileCoverImage = data.cover.coverPhoto.url;
                        else
                            localStorage['imagenCover'] = $rootScope.profileCoverImage = result.data.Picture;

                        // RELLENAR FICHA CLIENTE
                        $rootScope.cliente = result.data;

                        if ($rootScope.nextScreen == 'home'){

                            $state.go('menu.home');
                        }
                        else if ( $rootScope.nextScreen == 'goldcarClub' ) {

                            $state.go('menu.goldcarClub');
                        }
                        else{
                            if ( $rootScope.nextScreen == 'reserva'){
                                $rootScope.recogerDatosLogin(result.data);
                            }
                            if ( $rootScope.nextScreen == 'extras'){
                                $rootScope.recogerDatosLoginExtra(result.data);
                                $rootScope.getPromos();
                                $rootScope.getPremios();
                            }
                        }

                        $ionicLoading.hide();

                        if ( $rootScope.modalLogin ) {

                            $rootScope.modalLogin.hide();
                        }

                    }, function (error) {
                        $ionicLoading.hide();
                        $log.debug('Login en Goldcar a través de Google+ fallido, error: ', error);
                        $ionicPopup.alert({title: $filter("translate")($rootScope.translations.general.error), template: $filter("translate")($rootScope.translations.alertas.error_gplus)});
                        $rootScope.modalLogin.hide();
                    });
            },

            getGooglePlusData: function (accessToken) { // VER DATOS USUARIO GOOGLE +

                return $http.get('https://www.googleapis.com/plus/v1/people/me?access_token=' + accessToken)
                    .success(function (data, status, headers) {
                        console.log("getGooglePlusData", data);
                        return data;
                    });
            },

            GPlusLogout: function() { // This will clear the OAuth2 token.
                window.plugins.googleplus.logout(
                    function (msg) {
                        $log.debug('Logout msg: ', msg);
                    }
                );
            },

            GPlusDisconnect: function() { // This will clear the OAuth2 token, forget which account was used to login, and disconnect that account from the app. This will require the user to allow the app access again next time they sign in. Be aware that this effect is not always instantaneous. It can take time to completely disconnect.
                window.plugins.googleplus.disconnect(
                    function (msg) {
                        $log.debug('Disconnect msg: ', msg);
                    }
                );
            },

            ////////////////////////////////
            //	ACCESO MEDIANTE FACEBOOK  //
            ////////////////////////////////

            FBLogin: function() {
                console.log("FBLogin");
                var service = this;
                facebookConnectPlugin.login(["public_profile", "email"], function(success){
                    $log.debug('Login success: ', success);
                    service.FBUserData(success);
                }, function(error){
                    $ionicLoading.hide();
                    $log.debug('Login error: ', error);
                })
            },

            FBUserData: function(data) {
                console.log("FBUserData");
                var service = this;
                var width = 2000;
                var height = 2000;
                facebookConnectPlugin.api("/me?fields=email,first_name,gender,link,last_name,locale,name,timezone,updated_time,verified,cover,picture.height(" + height + ").width(" + width + ")", ["public_profile", "email"], function(result){
                    $log.debug('UserData success: ', result);
                    service.FBSendDataGoldcar(result, data.authResponse.accessToken);
                }, function(error){
                    $log.debug('UserData error: ', error);
                    service.FBLogin();
                })
            },

            FBSendDataGoldcar: function(data, accessToken) {
                console.log("FBSendDataGoldcar");
                var resolvedData = {
                    'accessToken': accessToken,
                    // 'age_range': data.age_range.min,
                    'email': data.email,
                    'first_name': data.first_name,
                    'gender': data.gender,
                    'link': data.link,
                    'last_name': data.last_name,
                    'locale': data.locale,
                    'name': data.name,
                    'nombreRS': 'facebook',
                    'picture': data.picture.data.url,
                    'timezone': data.timezone,
                    'updated_time': data.updated_time,
                    'usuarioIdent': data.id,
                    'verified': data.verified,
                    'referer': 'app'
                };

                $rootScope.resolvedDataRRSS = resolvedData;
                console.log($rootScope.resolvedDataRRSS);

                this.loginWithRRSS(resolvedData)
                    .then(function (result) {
                        $log.debug('Login en Goldcar a través de Facebook realizado con éxito, resultado: ', result);

                        localStorage['user'] = result.data.Email;
                        localStorage['redSocial'] = 'facebook';
                        localStorage['nombreUsuario'] = $rootScope.nombreUsuario = result.data.Nombre;
                        localStorage['apellidosUsuario'] = $rootScope.apellidosUsuario = result.data.Apellidos;
                        localStorage['puntosClub'] = $rootScope.puntosClub = result.data.Puntosclub;
                        localStorage['imagenPerfil'] = $rootScope.imagenPerfil = result.data.Picture;
                        localStorage['isLogged'] = $rootScope.isLogged = true;
                        $rootScope.datosUsuario = result.data;

                        Session.set({

                            "Lang": localStorage['lang'],
                            "nombre": $rootScope.datosUsuario.Nombre,
                            "apellidos": $rootScope.datosUsuario.Apellidos,
                            "user": $rootScope.datosUsuario.user,
                            "authenticated": $rootScope.datosUsuario.Authenticated
                        }).then(function (result) {

                        });

                        if(typeof data.cover !== 'undefined')
                            localStorage['imagenCover'] = $rootScope.profileCoverImage = data.cover.source;
                        else
                            localStorage['imagenCover'] = $rootScope.profileCoverImage = result.data.Picture;

                        // RELLENAR FICHA CLIENTE
                        $rootScope.cliente = result.data;

                        if ($rootScope.nextScreen == 'home'){
                            $state.go('menu.home');
                        }
                        else if ( $rootScope.nextScreen == 'goldcarClub' ) {

                            $state.go('menu.goldcarClub');
                        }
                        else{
                            if ( $rootScope.nextScreen == 'reserva'){
                                $rootScope.recogerDatosLogin(result.data);
                            }
                            if ( $rootScope.nextScreen == 'extras'){
                                $rootScope.recogerDatosLoginExtra(result.data);
                                $rootScope.getPromos();
                                $rootScope.getPremios();
                            }
                        }

                        $ionicLoading.hide();

                        if ( $rootScope.modalLogin ) {

                            $rootScope.modalLogin.hide();
                        }


                    }, function (error) {
                        $ionicLoading.hide();
                        $log.debug('Login en Goldcar a través de Facebook fallido, error: ', error);
                        this.FBLogout();
                        $ionicPopup.alert({title: $filter("translate")($rootScope.translations.general.error), template: $filter("translate")($rootScope.translations.alertas.error_facebook)});
                        $rootScope.modalLogin.hide();
                    });
            },

            FBSilentLogin: function() {
                console.log("FBSilentLogin");
                var deferred = $q.defer();
                var service = this;

                $ionicLoading.show({
                    template: '<div class="cargando"><i style="font-weight: bold; margin-bottom: 5px;">' + $filter("translate")($rootScope.translations.loading.entrando) + '</i><br><ion-spinner icon="bubbles" class="spinner-positive"></ion-spinner></div>'
                });

                facebookConnectPlugin.getLoginStatus(function(success){
                    $log.debug('SilentLogin success: ', success);
                    if(typeof success !== 'undefined' && typeof success.status !== 'undefined' && success.status != 'connected'){
                        service.FBLogin();
                    }
                    else{
                        service.FBUserData(success);
                        deferred.resolve(success);
                    }
                }, function(error){
                    service.FBLogin();
                    $log.debug('SilentLogin error: ', error);
                    deferred.reject(error);
                })
                return deferred.promise;
            },

            FBGetAccessToken: function() {
                console.log("FBGetAccessToken");
                facebookConnectPlugin.getAccessToken(function(success){
                    $log.debug('AccessToken success: ', success);
                }, function(error){
                    $log.debug('AccessToken error: ', error);
                })
            },

            FBLogout: function() {
                console.log("FBLogout");
                facebookConnectPlugin.logout(function(success){
                    $log.debug('Logout success: ', success);
                }, function(error){
                    $log.debug('Logout error: ', error);
                })
            },

            ///////////////////////////////
            //	ACCESO MEDIANTE TWITTER  //
            ///////////////////////////////

            // TwitterLogin: function() {
            //     var deferred = $q.defer();
            //     var consumerKey = "5Y00RxqTLuA41YcgoDqpuUOvv";
            //     var consumerSecretKey = "aa9QzbQjsTwQKTmSOqgnJux2a2CzPCJcPFrDjcmBHsMA3QivmO";

            //     $cordovaOauth.twitter(consumerKey, consumerSecretKey, {})
            //         .then(function (success) {
            //             $log.debug('Twitter success: ', success);
            //             deferred.resolve(success);
            //         }, function (error) {
            //             $log.debug('Twitter error: ', error);
            //             deferred.reject(error);
            //         }
            //     );

            //     return deferred.promise;
            // },

            ///////////////////////////////////
            //	AUTENTICACIÓN CONTRA GOLDCAR
            ////////////////////////////////////

            loginWithRRSS: function (data) {
                console.log("loginWithRRSS");
                console.log(data);
                // InfoToWidget.loginRRSS(null, null, data);
                return $http.post('https://www.goldcar.es/api/v1/login/rrss', data)
                    .success(function (data, status, headers) {
                        GoldcarService.getReservasActivas().then(function (result) {

                                if (result.data.length > 0) {
                                    $rootScope.someBooking = true;
                                    $rootScope.someBookingLoaded = true;
                                    return true;
                                }
                                reservasPosiblementeActivas();
                            },
                            function (error) {
                                $log.debug("Error cargando reservasActivas: " + error);
                                reservasPosiblementeActivas();
                            }
                        );
                        var reservasPosiblementeActivas = function(){

                            GoldcarService.getReservasAnteriores().then(function (result) {

                                    if(result.data.length > 0){
                                        result.data.forEach(function (reserva) {
                                            var now = new Date().getTime();
                                            var diffHoursRecogida = reserva.Pickupdate;
                                            var diffHoursEntrega = reserva.Dropoffdate;

                                            if(reserva.Estado == 2 || reserva.Estado == 5){
                                                if(now > diffHoursRecogida && now < diffHoursEntrega) {
                                                    $rootScope.someBooking = true;
                                                    $rootScope.someBookingLoaded = true;
                                                    return true;
                                                }
                                            }
                                        });
                                    }
                                },
                                function(error){
                                    $log.debug("Error con reservasAnteriores: " + error);
                                });
                        };
                    });
            }
        };
    });
