angular.module('goldcar').controller('AyudaCtrl', function($filter, $scope, $ionicModal, $ionicLoading, $cordovaInAppBrowser, JsonService, $rootScope, $log, $ionicPopup, $window, $ionicScrollDelegate) {

    //----------------------------------
    // CREAR MODAL DE LISTA FACTURAS
    //----------------------------------

    $scope.createListaFacturas = function(fn) {
        $ionicModal.fromTemplateUrl('templates/ayuda/listaFacturas.html', {

            vista: "Lista Facturas",
            scope: $rootScope,
            animation: 'slide-in-up'
        }).then(function(modalListaFacturas) {
            $rootScope.modalListaFacturas = modalListaFacturas;
            if(fn) fn();
        });
    };

    //----------------------------------
    // RECUPERAR FACTURAS
    //----------------------------------

    $rootScope.solFactura = [];

    $rootScope.solicitarFactura = function(){

        if ( typeof $scope.solFactura.codContrato === "undefined" || typeof $scope.solFactura.codCliente === "undefined" || $scope.solFactura.codContrato == "" || $scope.solFactura.codCliente == "" || $scope.solFactura.codContrato == null || $scope.solFactura.codCliente == null ) {

            $ionicPopup.alert({
                title: $filter("translate")($rootScope.translations.alertas.faltan_datos),
                template: $filter("translate")($rootScope.translations.alertas.factura_campos_vacios)
            });
        } else {

            var data = {
                "numContrato" : $scope.solFactura.codContrato,
                "numCliente" : $scope.solFactura.codCliente
            };

            $ionicLoading.show({

                template: '<div class="cargando"><i style="font-weight: bold;">' + $filter("translate")($rootScope.translations.loading.solicitando_facturas) + '</i><br><br><ion-spinner icon="bubbles" class="spinner-positive"></ion-spinner></div>'
            });

            JsonService.postSolicitarFacturas(data).then(function (result) {

                $rootScope.facturas = result.data.data.items;

                $log.debug(result.data.data.items.length);

                for ( var i = 0; i != $rootScope.facturas.length; i++ ) {

                    $rootScope.facturas[i].TotalFra = (($rootScope.facturas[i].TotalFra).toString()).substring(5, 0);
                }

                if ( result.data.data.totalCount == 0 ) {

                    $ionicPopup.alert({
                        title: $filter("translate")($rootScope.translations.alertas.atencion),
                        template: $filter("translate")($rootScope.translations.alertas.error_facturas)
                    });

                    $ionicLoading.hide();
                }
                else{
                    if ( typeof $rootScope.modalListaFacturas === "undefined" ) {

                        $ionicLoading.hide();
                        $rootScope.createListaFacturas();
                    } else {

                        $ionicLoading.hide();
                        $rootScope.modalListaFacturas.show();
                    }
                }

            }, function(error){
                $ionicLoading.hide();
                $log.debug("Error solicitando facturas: ", error);
            });
        }
    };

    //----------------------------------
    // DESCARGAR FACTURAS
    //----------------------------------

    $rootScope.descargarFactura = function(link){

        $ionicLoading.show({

            template: '<div class="cargando"><i style="font-weight: bold;">' + $filter("translate")($rootScope.translations.loading.buscando_factura) + '</i><br><br><ion-spinner icon="bubbles" class="spinner-positive"></ion-spinner></div>'
        });

        $cordovaInAppBrowser.open(link, '_system', {location: 'yes'});

        $ionicLoading.hide();
    };

    //----------------------------------
    // ENVIAR FACTURAS POR CORREO
    //----------------------------------

    $rootScope.enviarFactura = function (link){

        $scope.data = {};

        $ionicPopup.show({
            template: '<input type="email" ng-model="data.emailTo" style="border: 1px solid #eeeeee; text-align: center; padding: 5px">',
            title: $filter("translate")($rootScope.translations.ayuda.destinatario)+": ",
            scope: $scope,
            buttons: [
                {
                    text: $filter("translate")($rootScope.translations.general.cancelar),
                    type: 'button-assertive buttonCancelar'
                },
                {
                    text: '<b>' + $filter("translate")($rootScope.translations.general.enviar) + '</b>',
                    type: 'button-positive buttonCancelar',
                    onTap: function(e) {
                        if (!$scope.data.emailTo) {
                            e.preventDefault();
                        } else {
                            var text = "mailto:" + $scope.data.emailTo + "?subject=&body=" + link;
                            $window.location.href = decodeURIComponent(text);
                        }
                    }
                }
            ]
        });
    }

    // REPORTAR INCIDENCIA //
    $rootScope.reportarIncidencia = function () {
        if($rootScope.language == 'es') var url = encodeURI("https://www.goldcar.es/incidencias");
        else var url = encodeURI("https://www.goldcar.es/" + $rootScope.config.mapeo_api[$rootScope.language] + "/incidencias");
        var target = '_blank';
        var options = 'hideurlbar=yes';

        cordova.InAppBrowser.open(url, target, options);
    }

    $scope.mostrarInfoFactura = function () {
        if($rootScope.language == 'es') $scope.imgContrato = 'contrato_es.png';
        else $scope.imgContrato = 'contrato_en.png';

        $ionicModal.fromTemplateUrl('templates/ayuda/infoFactura.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function(modal) {
            $scope.modalInfoFactura = modal;
            modal.show();
        });
    };

    $scope.cerrarModalInfoFactura = function(){
        $scope.modalInfoFactura.remove();
    };

    $scope.zoomImage = function () {
        $ionicScrollDelegate.$getByHandle('scroll-info-factura').zoomBy(1, true);
    };

 });
