angular.module('goldcar').controller('DetallesReservaCtrl', function($scope, GoldcarService, JsonService, $ionicModal, $rootScope, $ionicLoading, $filter, $ionicPopup, $log, Session) {

    //--------------------------
    // MODAL CONDICIONES
    //--------------------------

    $ionicModal.fromTemplateUrl('templates/detalles_reserva/condiciones_alquiler.html', {

        vista: "/" + $rootScope.language + "/mygoldcar/tyc",
        scope: $rootScope,
        animation: 'slide-in-up'
    }).then(function(modalCondicAlquiler) {

        $rootScope.modalCondicAlquiler = modalCondicAlquiler;
    });

    $scope.abrirCondiciones = function(reserva) {

        $ionicLoading.show();

        var oficina = reserva.Pickupplace.Codigo;
        var fechaRecogida = moment(reserva.Pickupdate).format('YYYY-MM-DD');
        var fechaEntrega = moment(reserva.Dropoffdate).format('YYYY-MM-DD');
        var grupo = reserva.Grupo.Codigo;

        GoldcarService.getCondicionesServicio(oficina, fechaRecogida, fechaEntrega, grupo).then(function (result) {
            $rootScope.condicionesActivas = result.data;
            $ionicLoading.hide();
        }, function (error) {
            $log.debug("Error trayendo las condiciones del servicio: ", error);
            $ionicLoading.hide();
        });

        $rootScope.modalCondicAlquiler.show();
    };

    $scope.cancelarReserva = function(reserva) {

        var cancelarPopup = $ionicPopup.show({

            template: '',
            title: $filter("translate")($rootScope.translations.detalles_reserva.cancelar_reserva),
            subTitle: $filter("translate")($rootScope.translations.detalles_reserva.confirmacion_cancelar_reserva) ,
            scope: $rootScope,
            cssClass: 'multi-popup',
            buttons: [
                {
                    text: $filter("translate")($rootScope.translations.general.no),
                    type: 'buttonCancelar',
                    onTap: function () {
                        $ionicPopup.show().close();
                    }
                },
                {
                    text: '<b>' + $filter("translate")($rootScope.translations.general.si) + '</b>',
                    type: 'buttonAceptar',
                    onTap: function() {
                        $ionicLoading.show();

                        GoldcarService.cancelarReserva(reserva).then(function (result) {
                            $ionicLoading.hide();
                            setTimeout(function () {

                                $ionicPopup.alert({ title: $filter("translate")($rootScope.translations.datos.estado_reserva), cssClass: 'single-popup',template: $filter("translate")($rootScope.translations.alertas.reserva_cancelada)});
                            });
                            cancelarPopup.close();
                            $rootScope.modalDetallesRes.hide();

                            $rootScope.pocoTiempo();
                        }, function (error) {
                            $log.debug("Error cancelando la reserva: ", error);
                            cancelarPopup.close();
                            $ionicLoading.hide();
                            $ionicPopup.alert({ title: $filter("translate")($rootScope.translations.general.error), template: $filter("translate")($rootScope.translations.alertas.error_autenticacion_texto)});
                        });
                    }
                }
            ]
        });

        setTimeout(function () {

            angular.element( document.querySelectorAll( '.buttonCancelar' ) ).attr('type', 'button');
        }, 200);
    };

    $scope.openPassbook = function (idReserva) {
        var patron = $rootScope.config.patronPassbook;

        var encripted = sha256(idReserva + $rootScope.datosUsuario.Apellidos + $rootScope.reservaSeleccionada.Bonocliente + patron);
        var url = 'https://www.goldcar.es/passbook/gen/idreserva/' + idReserva + '/e/' + encripted;

        $log.debug("Encripted url passbook: ", url);

        cordova.InAppBrowser.open(encodeURI(url), '_blank');
    };

    $scope.openReservaPassbook = function (urlPassbook) {

        cordova.InAppBrowser.open(encodeURI(urlPassbook), '_system');
    };

    // CONTRATAR EXPRESS CHECKIN

    $scope.disponibleExpress = true;
    $scope.extraExpressService = "";
    $scope.extras_render = [];

    $scope.comprobarExpressCheckIn = function (reserva) {

        console.log(reserva);

        var noDisponible = 0;

        if ( !reserva.Modificable ) {

            noDisponible = 1;
        }

        if ( noDisponible == 0 ) {

            for ( var i = 0; i < reserva.Extras.length; i++ ) {

                if ( reserva.Extras[i].Codigo == 'ExpressService' ) {

                    noDisponible = 1;
                    break;
                }
            }
        }

        if ( noDisponible == 0 ) {

            var date = new Date();
            date = date.toISOString();

            if ( parseInt(reserva.PickupdateTime[0]) <= 9 && reserva.PickupdateTime[1] == ":" ) {

                reserva.PickupdateTime = 0 + reserva.PickupdateTime;
                console.log(reserva.PickupdateTime);
            }

            Session.set(angular.merge({}, $scope.session, {

                "Lang": $rootScope.language,
                "dropoffplace": reserva.Dropoffplace.Codigo,
                "pickupplace": reserva.Pickupplace.Codigo,
                "dropoffdate": reserva.Dropoffdate.format('YYYY-MM-DD'),
                "dropoffdate_format": (reserva.Dropoffdate.format('DD MMM').slice(0, 6)).toUpperCase(),
                "pickupdate": reserva.Pickupdate.format('YYYY-MM-DD'),
                "pickupdate_format": (reserva.Pickupdate.format('DD MMM').slice(0, 6)).toUpperCase(),
                "dropofftime": reserva.DropoffdateTime,
                "pickuptime": reserva.PickupdateTime,
                "group": reserva.Grupo.Codigo,
                "timestamp": date
            })).then(function (result) {

                GoldcarService.getDisponibilidadConGrupo(reserva.Idreserva).then(function (data) {

                    $scope.grupoReservaSelec = data.data;

                    if ( !data.data.error ) {

                        for ( var j = 0; j < data.data.extras.length; j++ ) {

                            if ( data.data.extras[j].alias == 'ExpressService' ) {
                                
                                $scope.precioExpress = data.data.extras[j].precio_total;
                                $scope.divisaExpress = data.data.divisa;

                                if ( data.data.extras[j].disponible != true ) {

                                    noDisponible = 1;
                                    break;
                                } else if ( data.data.extras[j].disponible == true ) {

                                    $scope.extraExpressService = data.data.extras[j];
                                    break;
                                }

                            } else {

                                noDisponible = 1;
                            }
                        }
                    } else {

                        noDisponible = 1;
                    }
                })
            });
        }

        if ( noDisponible > 0 ) {

            $scope.disponibleExpress = false;
        } else {

            $scope.disponibleExpress = true;
        }

        // CREEMOS QUE SE HA QUITADO EL EXPRESS CHECKIN DE SU OFERTA
        $scope.disponibleExpress = false;
    };

    $rootScope.contratarExpress = function (reserva) {

        var confirmPopup = $ionicPopup.confirm({
            title: $filter("translate")($rootScope.translations.detalles_reserva.title_express),
            template: $filter("translate")($rootScope.translations.detalles_reserva.seguro_express),
            buttons: [
                {
                    text: $rootScope.translations.general.cancelar,
                    type: 'button-assertive',
                    onTap: function () {
                    }
                },
                {
                    text: 'Contratar',
                    type: 'button-positive',
                    onTap: function () {

                        $ionicLoading.show();

                        if ( $scope.disponibleExpress ) {

                            if ( $scope.extraExpressService != "" ) {

                                reserva.Extras.push($scope.extraExpressService);
                            }

                            var reservaModificada = reserva;

                            reserva.Extras.push("ExpressService");

                            reservaModificada.pickupplace = reserva.Pickupplace.Codigo;
                            reservaModificada.pickupdate = reserva.Pickupdate.format('YYYY-MM-DD');
                            reservaModificada.pickuptime = reserva.PickupdateTime;
                            reservaModificada.dropoffplace = reserva.Dropoffplace.Codigo;
                            reservaModificada.dropoffdate = reserva.Dropoffdate.format('YYYY-MM-DD');
                            reservaModificada.dropofftime = reserva.DropoffdateTime;
                            reservaModificada.group = reserva.Grupo.Codigo;
                            reservaModificada.extras_seleccionados = [];

                            reservaModificada.extras_seleccionados.push("ExpressService");

                            GoldcarService.putReserva(reservaModificada).then(function (result) {
                                $ionicLoading.hide();
                                $scope.disponibleExpress = false;
                                $rootScope.modalDetallesRes.remove();
                                $rootScope.pocoTiempo();
                            });
                        } else {

                            $ionicPopup.alert({ title: $filter("translate")($rootScope.translations.alertas.atencion), template: $filter("translate")($rootScope.translations.detalles_reserva.no_express)});
                        }
                    }

                }
            ]
        });
    };

    // ------------------------- //
    // COMPROBAR TICKETS OFICINA //
    // ------------------------- //
    
    // $scope.showTicketSystem = false;

    // $scope.comprobarSistemaTickets = function(){
    //     JsonService.getOffice($rootScope.reservaSeleccionada.Pickupplace.Codigo).then(function(result){
    //         // console.log(result.data.office.ticketSystem);
    //         if(result.data.office.ticketSystem != null && result.data.office.ticketSystem != ''){
    //             $scope.showTicketSystem = true;
    //         }
    //     })
    // }
});
