angular.module('goldcar').controller('LocalizacionesCtrl', function($filter, $scope, $ionicModal, $rootScope, $ionicLoading, $cordovaSocialSharing, $log, $ionicPopup) {

    var latitud = "";
    var longitud = "";
    $rootScope.nuevaLocalizacion = [];
    $rootScope.localizGuardadas = [];
    $rootScope.hayLocalizaciones = false;

    //----------------------------------
    // CREAR DB LOCALIZACIONES SQLITE
    //----------------------------------

    $rootScope.getLocalizaciones = function() {

        if (typeof window.sqlitePlugin !== 'undefined' && window.sqlitePlugin) {
            $scope.db = window.sqlitePlugin.openDatabase({name: "GoldcarAPP.db", location: 'default', androidLockWorkaround: 1});
            var queryCreateTable = "CREATE TABLE IF NOT EXISTS localizaciones (id INTEGER PRIMARY KEY AUTOINCREMENT, nombre TEXT, latitud TEXT, longitud TEXT)";
            
            $scope.db.transaction(function(tx) {
                tx.executeSql(queryCreateTable);
            });
    
            var querySelect = "SELECT * FROM localizaciones";
            $scope.db.transaction(function(tx) {
                $ionicLoading.show();

                tx.executeSql(querySelect, [], function(tx, result) {
                    if ( result.rows.length > 0 ) {
                        $rootScope.hayLocalizaciones = true;
                        for ( var i = 0; i != result.rows.length; i++) {
                            $rootScope.localizGuardadas[i] = result.rows.item(i);
                        }
                        $ionicLoading.hide();
                    } else {
                        $ionicLoading.hide();
                    }
                });
            });
        }
    };

    //---------------------------
    // MODAL NUEVA LOCALIZACION
    //---------------------------

    $scope.createModalNuevaLocalizacion = function () {

        if ( typeof $rootScope.modalNuevaLocalizacion === 'undefined' ) {

            $ionicModal.fromTemplateUrl('templates/localizaciones/nueva_localizacion.html', {

                vista: "/" + $rootScope.language + "/gps/new",
                scope: $rootScope,
                animation: 'slide-in-up'
            }).then(function(modalNuevaLocalizacion) {

                $ionicLoading.hide();
                $rootScope.modalNuevaLocalizacion = modalNuevaLocalizacion;
                modalNuevaLocalizacion.show();
            });
        } else {

            $ionicLoading.hide();
            $rootScope.modalNuevaLocalizacion.show();
        }
    };

    //--------------------------
    // CREAR NUEVA LOCALIZACION
    //--------------------------

    $rootScope.nuevaUbicacion = function() {

        $ionicLoading.show();

        var opciones = {
            timeout: 5000,
            maximumAge: 0,
            enableHighAccuracy: false
        };

        navigator.geolocation.getCurrentPosition(successGeolocation, errorGeolocation, opciones);

        function successGeolocation(posicion) {

            latitud = posicion.coords.latitude;
            longitud = posicion.coords.longitude;

            $rootScope.coordsNuevaLoc = {"latitud": latitud, "longitud": longitud};
            $scope.createModalNuevaLocalizacion();
        }

        function errorGeolocation(error) {

            if ( error.code == 1 || error.code == 3 ) {

                $ionicPopup.alert({ title: $filter("translate")($rootScope.translations.general.error), template: $filter("translate")($rootScope.translations.alertas.error_deshabilitada_geoloc) });
            } else {

                $log.error("Error localizaciones", error);
            }
            $ionicLoading.hide();
        }


    }

    //--------------------------
    // BORRAR LOCALIZACION
    //--------------------------

    $rootScope.borrarUbicacion = function(id, index) {

        $rootScope.checkLocalizaciones = [];
        $rootScope.checkLocalizaciones.borrar = false;

        $ionicPopup.show({

            template: '<md-checkbox ng-model="checkLocalizaciones.borrar">' + $filter("translate")($rootScope.translations.localizaciones.borrar_localizacion_titulo) + '</md-checkbox>',
            title: $filter("translate")($rootScope.translations.localizaciones.borrar_localizacion1),
            subTitle: $filter("translate")($rootScope.translations.localizaciones.borrar_localizacion2),
            scope: $rootScope,
            buttons: [
                {
                    text: $filter("translate")($rootScope.translations.general.cancelar),
                    type: 'button-assertive',
                    onTap: $ionicPopup.show().close()
                },
                {
                    text: '<b>' + $filter("translate")($rootScope.translations.general.eliminar) + '</b>',
                    type: 'button-positive',
                    onTap: function(e) {
                        if ( $rootScope.checkLocalizaciones.borrar != true ) {

                            e.preventDefault();
                        } else {

                            eliminarUbicacion(id, index);
                            $ionicPopup.show().close();
                        }
                    }
                }
            ]
        });

        function eliminarUbicacion(id, index) {

            var query = "DELETE FROM localizaciones WHERE id = ?";
            $scope.db.transaction(function(tx) {

                tx.executeSql(query, [id], function(tx, res) {

                    $rootScope.localizGuardadas.splice(index, 1);
                });
            });

            $rootScope.getLocalizaciones();
        }
    };

    //------------------------------
    // GUARDAR NUEVA LOCALIZACION
    //------------------------------

    $rootScope.guardarUbicacion = function(nombre) {

        if ( nombre == "" || typeof nombre == "undefined") {

            $ionicPopup.alert({ title: $filter("translate")($rootScope.translations.alertas.atencion), template: $filter("translate")($rootScope.translations.localizaciones.nombre_localizacion) });
        } else {

            var queryInsert = "INSERT INTO localizaciones (nombre, latitud, longitud) VALUES (?,?,?)";
            $scope.db.transaction(function(tx) {

                tx.executeSql(queryInsert, [nombre, $rootScope.coordsNuevaLoc.latitud, $rootScope.coordsNuevaLoc.longitud], function(tx, res) {
                    $log.debug("query de localizaciones satisfactoria -> ubicacion guardada correctamente: ", res);
                });
            });

            $rootScope.nuevaLocalizacion.nombreLocalizacion = "";
            $rootScope.modalNuevaLocalizacion.hide();
        }

        $rootScope.getLocalizaciones();
    };

    //------------------------------
    // COMPARTIR UBICACION
    //------------------------------

    $scope.compartirUbicacion = function(index) {

        $ionicLoading.show();

        var link = 'http://www.google.com/maps/place/' + $rootScope.localizGuardadas[index].latitud + ',' + $rootScope.localizGuardadas[index].longitud + '(' + $rootScope.localizGuardadas[index].nombre + ')';

        $cordovaSocialSharing
            .share(null, null, null, link)
            .then(function(result) {

            }, function(err) {

                $ionicPopup.alert({ title: $filter("translate")($rootScope.translations.general.error), template: $filter("translate")($rootScope.translations.alertas.error_conexion) });
                $log.debug("No se ha podido ejecutar cordovaSocialSharing: ", err);
            });

        $ionicLoading.hide();
    };

    //------------------------------
    // IR A LOCALIZACION
    //------------------------------

    $scope.irUbicacion = function(index) {

        launchnavigator.navigate($rootScope.localizGuardadas[index].latitud + ',' + $rootScope.localizGuardadas[index].longitud).then(function() {

        }, function (err) {
            $log.debug("No se ha podido ejecutar launchNavigator: ", err);
        });
    };
});
