angular.module('goldcar').controller('DetallesOficinaCtrl', function ($filter, $rootScope, $scope, $ionicLoading, $ionicModal, $ionicPopup, $ionicSlideBoxDelegate, $cordovaInAppBrowser, JsonService, GoldcarService, $log, $state) {

    //----------------------------------
    // CREAR MODAL DE DATOS DE OFICINA
    //----------------------------------

    $scope.createModalDetalleOficina = function (fn) {
        $ionicModal.fromTemplateUrl('templates/oficinas/detalles_oficina.html', {
            vista: "Detalles Oficina " + $rootScope.oficinaSeleccionada.code,
            id: "office_details",
            scope: $rootScope,
            animation: 'slide-in-up'
        }).then(function (modalDetalleOficina) {
            $rootScope.modalDetalleOficina = modalDetalleOficina;
            if (fn) fn();
        });
    };

    //----------------------------
    // CREAR MODAL HORARIOS
    //----------------------------

    $scope.createModalHorariosOficina = function (fn) {
        $ionicModal.fromTemplateUrl('templates/oficinas/horarios.html', {
            vista: "Horarios Oficina " + $rootScope.oficinaSeleccionada.code,
            scope: $rootScope,
            animation: 'slide-in-up'
        }).then(function (modalHorariosOficina) {
            $rootScope.modalHorariosOficina = modalHorariosOficina;
            if (fn) fn();
        });
    };

    //----------------------------------
    // OBTENER DATOS BLOG
    //----------------------------------

    $scope.cargarDescubre = function (codigoOficina, codigoPais) {
        JsonService.getDescubreData(codigoOficina).then(function (descubreDataOficina) {
            $scope.descubreData = descubreDataOficina;
            JsonService.getDescubreData(codigoPais).then(function (descubreDataPais) {
                descubreDataPais.forEach(function (banner) {
                    var contieneOficina = false;
                    banner.tags.forEach(function (tag) {
                        if (tag == codigoOficina) {
                            contieneOficina = true;
                        }
                    });
                    if (!contieneOficina) {
                        $scope.descubreData.push(banner);
                    }
                });
                $ionicSlideBoxDelegate.update();
            },
                function (error) {
                    $log.debug("Error trayendo datos de getDescubreDataPais: " + error);
                }
            );
            $ionicSlideBoxDelegate.update();
        },
            function (error) {
                $log.debug("Error trayendo datos de getDescubreDataOficina: " + error);
            }
        );
    };

    //----------------------------------
    // ABRIR OFICINA DESEADA
    //----------------------------------

    $rootScope.abrirOficina = function (codigoOficina) {

        $ionicLoading.show();
        var office = JsonService.getOfficeByCode(codigoOficina);

        if (codigoOficina) {
            GoldcarService.getDetallesOficina(codigoOficina).then(function (result) {
                $scope.oficinaHorario = result.data.Horario;
                $rootScope.oficinaHorario = result.data.Horario;
            });
        }

        if (office) {
            $rootScope.oficinaSeleccionada = office;
            $scope.createModalDetalleOficina(function () {
                $ionicLoading.hide();
                $rootScope.modalDetalleOficina.show();
            });
            if (office.phone) {

                var encontrado = office.phone.search("<br>");
                if (encontrado >= 0) {
                    office.phone = office.phone.split("<br>");
                    office.variousPhones = true;
                }
            }
        }
        else {
            $ionicLoading.hide();
            $ionicPopup.alert({
                title: $filter("translate")($rootScope.translations.alertas.atencion),
                template: $filter("translate")($rootScope.translations.alertas.error_oficina)
            });
        }
    };

    //----------------------------------
    // OBTENER EL TIEMPO
    //----------------------------------

    $scope.getTiempo = function (Codigoweather) {
        GoldcarService.getTiempoOficina(Codigoweather).then(function (result) {
            $scope.tiempo = result.data;
        },
            function (error) {
                $log.debug("Error con getTiempo: " + error);
            }
        );
    };

    //----------------------------------
    // FUNCIONALIDAD EXTRA
    //----------------------------------

    $scope.abrirReservar = function () {
        if ($rootScope.config.reservarEnWeb) {
            var url = 'https://www.goldcar.es/reservas/disponibilidad/#pickupplace/' + $rootScope.oficinaSeleccionada.code + '/referer/APP';

            $cordovaInAppBrowser.open(url, '_system', { location: 'yes' })
                .then(function (event) {
                })
                .catch(function (event) {
                });
        }
        else {
            $rootScope.modalOficinas.hide();
            $rootScope.modalDetalleOficina.remove();
            $rootScope.seleccionaOficinaRecogida($rootScope.oficinaSeleccionada.code, $rootScope.oficinaSeleccionada.fullname, $rootScope.oficinaSeleccionada.name, $rootScope.oficinaSeleccionada.type_goldcar, $rootScope.oficinaSeleccionada.bus, null, 'oficinas');
        }
    };

    $scope.abrirEnlace = function (url) {
        $cordovaInAppBrowser.open(url, '_system', { location: 'yes' })
            .then(function (event) {
            })
            .catch(function (event) {
            });
    };

    $scope.abrirQueHacer = function (codigoOficina) {
        var lang = 'en-GB';
        if ($rootScope.config.guidego_maplangs[$rootScope.language])
            lang = $rootScope.config.guidego_maplangs[$rootScope.language];

        var url = "https://www.guidego.com/widgets/ticket/?currency=EUR&partnerWL=goldcar&language=" + lang + "&dest=" + codigoOficina;

        $cordovaInAppBrowser.open(url, '_system', { location: 'yes' })
            .then(function (event) {
            })
            .catch(function (event) {
            });
    };

    $scope.abrirHorarios = function (horario) {
        console.log(horario);
        $rootScope.horarioOficinaSeleccionado = horario;
        $scope.createModalHorariosOficina(function () {
            $rootScope.modalHorariosOficina.show();
        });
    };
});